/*****************************************************************************
 * Package: gov.nih.nlm.mms
 * Object:  AboutAction.java
 *
 *
 ****************************************************************************/
package gov.nih.nlm.mms;

import java.awt.Frame;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

/**
 * This class is used for the <code>Help->About</code> menu of the
 * {@link MetamorphoSysGUI}.
 * 
 * @author Brian Carlsen, Deborah Shapiro
 */
public class AboutAction extends AbstractAction {

  private String title;

  private Frame owner;

  //
  // Constructor
  //

  /**
   * Instantiates an {@link AboutAction}.
   * 
   * @param title
   *          {@link String}
   * @param owner {@link Frame}
   */
  public AboutAction(String title, Frame owner) {
    super();
    this.title = title;
    this.owner = owner;
    // configure action
    putValue(Action.NAME, title);
    putValue(Action.SHORT_DESCRIPTION, title);
  }

  /**
   * Opens a small dialog informing the user about the application.
   * 
   * @param e
   *          an {@link java.awt.event.ActionEvent}
   */
  public void actionPerformed(ActionEvent e) {
    MMSToolkit.notifyUser(owner, "UMLS Release: "
        + MMSToolkit.getReleaseVersion() + "\n" + "Description: "
        + MMSToolkit.getReleaseDescription() + "\n" + "UMLS Release Date: "
        + MMSToolkit.getReleaseDate() + "\n" + "Build Date: "
        + MMSToolkit.getBuildDate() + "\n" + "MetamorphoSys Version: "
        + MMSToolkit.getApplicationVersion() + "\n"
        + "MetamorphoSys Build Date: " + MMSToolkit.getApplicationBuildDate(),
        title);
  }

}
