/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms;
 * Interface:  AdvancedOptionsGUI.java
 *
 *************************************************************************/
package gov.nih.nlm.mms;

import gov.nih.nlm.swing.FontSizeManager;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

/**
 * This dialog is used to configure the application level parameters. These are
 * configuration options that affect all filters or the application as a whole.
 * 
 * @author Deborah Shapiro
 * @see AdvancedOptionsAction
 */
public class AdvancedOptionsGUI extends JDialog implements ActionListener {

  //
  // Fields
  //
  private JCheckBox automatic_selection = null;

  private JButton jdone = null;

  private UserConfiguration config = null;

  //
  // Constructor
  //

  /**
   * Instantiates a {@link AdvancedOptionsGUI} from the specified
   * {@link UserConfiguration}. The constructor does not make it visible.
   * 
   * @param config
   *          the {@link UserConfiguration} modified by this GUI
   */
  public AdvancedOptionsGUI(UserConfiguration config) {
    super(MMSToolkit.getGui());
    this.config = config;
    setModal(true);
    setTitle("MetamorphoSys Options");
    initialize();
  };

  /**
   * Initializes GUI view.
   */
  private void initialize() {

    // Configure the GUI.
    getContentPane().setLayout(new BorderLayout());
    JPanel data_panel = new JPanel(new GridBagLayout());
    JPanel done_panel = new JPanel(new GridBagLayout());
    JPanel complete_panel = new JPanel(new BorderLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.fill = GridBagConstraints.BOTH;
    constraints.insets = new Insets(2, 2, 2, 2);

    // Auto Select checkbox
    constraints.gridy = 0;
    constraints.gridwidth = 3;
    constraints.gridy++;
    automatic_selection = new JCheckBox("Auto Select Related Items", false);
    automatic_selection.setSelected(config.getAutomaticSelection());
    data_panel.add(automatic_selection, constraints);

    // "Done" button
    constraints.gridx = 0;
    constraints.gridwidth = 1;
    constraints.weightx = 1.0;
    constraints.gridy++;
    jdone = new JButton("Done");
    jdone.addActionListener(this);
    done_panel.add(jdone, constraints);

    // Assemble panels and pack the dialog
    complete_panel.add(data_panel, BorderLayout.NORTH);
    complete_panel.add(done_panel, BorderLayout.SOUTH);
    EtchedBorder etched_border = new EtchedBorder(EtchedBorder.RAISED);
    complete_panel.setBorder(new CompoundBorder(etched_border, new EmptyBorder(
        10, 10, 10, 10)));

    getRootPane().setDefaultButton(jdone);
    getContentPane().add(complete_panel, BorderLayout.CENTER);
    FontSizeManager.adjustFontSize(this);
    this.pack();
    this.setLocationRelativeTo(null);
  };

  //
  // Methods
  //

  /**
   * Handles the "Done" button. It commits the changes to the configuration
   * object and disposes of the GUI.
   * 
   * @param e
   *          an {@link java.awt.event.ActionEvent} from the "Done" button.
   */
  public void actionPerformed(ActionEvent e) {
    setValues();
    setVisible(false);
    this.dispose();
  }

  /**
   * Translates GUI selections into configuration settings.
   */
  private void setValues() {
    config.setAutomaticSelection(automatic_selection.isSelected());
  }

} // End of AdvancedOptionsGUI class

