/**
 * Runs a batch MetamorphoSys.  Properties are:
 * -Dmmsys.config = the configuration file to use
 * -Dmmsys.dir= MMSYS installation directory
 * -Dinput.dir= the Metathesarus input directory
 * -Doutput.dir= the output directory
 */
package gov.nih.nlm.mms;
import java.io.File;

public class BatchMetamorphoSys {

    public static void main (String[] argv) {
	String mmsysdir=null;
	String inputdir=null;
	String outputdir=null;
	String releasedat=null;
	String s;

	ApplicationConfiguration appconfig = null;
	UserConfiguration userconfig = null;

        if ((s = System.getProperty("input.dir")) != null) {
	    try {
		File f = new File(s);
		if (f.exists() && f.isDirectory()) {
		    inputdir = s;
		    releasedat = inputdir + File.separator + "release.dat";
		    File f2 = new File(releasedat);
		    if (!(f2.exists() && f2.isFile() && f2.canRead())) {
			throw(new Exception("ERROR: Cannot find or read release.dat at: " + releasedat));
		    }
		} else {
		    throw(new Exception("ERROR: input directory: " + s + " does not exist!"));
		}
	    } catch (Exception e) {
		System.err.println(e);
		System.exit(2);
	    }
	} else {
	    System.err.println("The input.dir property must be specified.");
	    System.exit(2);
	}

        if ((s = System.getProperty("output.dir")) != null) {
	    try {
		File f = new File(s);
		if (f.exists() && f.isDirectory()) {
		    outputdir = s;
		} else {
		    throw(new Exception("ERROR: output directory: " + s + " does not exist!"));
		}
	    } catch (Exception e) {
		System.err.println(e);
		System.exit(2);
	    }
	} else {
	    System.err.println("The output.dir property must be specified.");
	    System.exit(2);
	}

        if ((s = System.getProperty("mmsys.config")) != null) {
	    try {
		File f = new File(s);
		if (f.exists()) {
		    userconfig = new UserConfiguration(f);
		} else {
		    throw(new Exception("ERROR: configuration file: " + s + " does not exist!"));
		}
	    } catch (Exception e) {
		System.err.println(e);
		System.exit(2);
	    }
	} else {
	    System.err.println("The mmsys.config property must be specified.");
	    System.exit(2);
	}

        if ((s = System.getProperty("mmsys.dir")) != null) {
	    try {
		mmsysdir = s;
		String c = s + File.separator + "config" + File.separator + "mmsys.a.prop";
		File f = new File(c);
		if (f.exists()) {
		    appconfig = new ApplicationConfiguration(f);
		} else {
		    throw(new Exception("ERROR: MMSYS application configuration: " + c + " does not exist!"));
		}
	    } catch (Exception e) {
		System.err.println(e);
		System.exit(2);
	    }
	} else {
	    System.err.println("The mmsys.dir property must be specified.");
	    System.exit(2);
	}

	if (appconfig == null || userconfig == null || releasedat == null) {
	    System.err.println("ERROR: some config properties are null");
	    System.exit(2);
	}

        try {
            userconfig.setSourcePaths(new String[] { inputdir });
            userconfig.setSubsetDirectory(outputdir);
            appconfig.setConfigDirectory(mmsysdir);
	    MMSToolkit.setReleaseDatLocation(releasedat);
            MetamorphoSys mms = new MetamorphoSys(appconfig, userconfig);
            mms.initializeConfigurables();
            mms.subset();
        } catch (Exception e) {
            System.err.println("ERROR in MetamorphoSys - exiting");
            e.printStackTrace();
	    System.exit(2);
        }
	System.exit(0);
    }
}
