/**************************************************************************
 *
 * Package:	gov.nih.nlm.mms
 * Object:	DefaultSubset.java
 *
 **************************************************************************/
package gov.nih.nlm.mms;



/**
 * Represents a default initial configuration selected by a user.
 *
 * @author Deborah Shapiro, Brian Carlsen
 */
public class DefaultSubset {

  //
  // Private Fields
  //
  private String id;
  private String name;
  private String dsc;
  private String config_file = null;

  //
  // Constructors
  //
  /**
   * Instantiates an empty {@link DefaultSubset}.
   */
  public DefaultSubset() {};

  //
  // Methods
  //

  /**
   * Returns the identifier of the {@link DefaultSubset}.
   * @return the identifier of the {@link DefaultSubset}
   */
  public String getId() {
    return id;
  }

  /**
   * Sets the identifier of the {@link DefaultSubset}.
   * @param id the identifier of the {@link DefaultSubset}
   */
  public void setId(String id) {
    this.id = id;
  }

  /**
   * Returns the name of the {@link DefaultSubset}.
   * @return the name of the {@link DefaultSubset}
   */
  public String getName() {
    return name;
  }

  /**
   * Sets the name of the {@link DefaultSubset}.
   * @param name the name of the {@link DefaultSubset}
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * Returns the name of the configuration file to use.
   * @return the name of the configuration file to use
   */
  public String getConfigurationFilename() {
    return config_file;
  }

  /**
   * Sets the name of the configuration file to use.
   * @param name the name of the configuration file to use
   */
  public void setConfigurationFilename(String name) {
    config_file = name;
  }

  /**
   * Returns the description of the {@link DefaultSubset}.
   * @return the description of the {@link DefaultSubset}
   */
  public String getDescription() {
    return dsc;
  }

  /**
   * Sets the description of the {@link DefaultSubset}.
   * @param dsc the description of the {@link DefaultSubset}
   */
  public void setDescription(String dsc) {
    this.dsc = dsc;
  }

  /**
   * Dump for debugging.
   * @return {@link String}
   */
  protected String dump() {
   return "Default Subset " + id + ", " + name + ", " + dsc + ", " + config_file;
  }

}