/****************************************************************************
 *
 * Package: gov.nih.nlm.mms
 * Object:  ExitAction.java
 *
 *****************************************************************************/
package gov.nih.nlm.mms;

import gov.nih.nlm.mms.browser.RRFBrowserGUI;

import java.awt.Event;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;

import javax.swing.Action;
import javax.swing.KeyStroke;

/**
 * Exits the application. Used for the <code>File -> Exit</code> menu of
 * {@link MetamorphoSysGUI}.
 * 
 * @author Brian Carlsen, Deborah Shapiro
 */
public class ExitAction extends SaveConfigAction {

  /**
   * Instantiates an {@link ExitAction}
   * 
   * @param mms
   *          the {@link MetamorphoSys} application object
   */
  public ExitAction(MetamorphoSys mms) {
    super(mms, MMSToolkit.getGui());
    putValue(Action.NAME, "Exit");
    putValue(Action.SHORT_DESCRIPTION, "Exit");
    putValue(Action.MNEMONIC_KEY, new Integer((int) 'q'));
    putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke('Q',
    		Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
  }

  /**
   * Exits the application after prompting user to save current configuration.
   * 
   * @param e
   *          the {@link ActionEvent}
   */
  public void actionPerformed(ActionEvent e) {

    UserConfiguration user_configuration = mms.getUserConfiguration();
    if (user_configuration.hasDataChanged()) {
      if (MMSToolkit.confirmRequest(MMSToolkit.getGui(),
          "You have made changes to this configuration.\n"
              + "Would you like to save the changes?")) {
        super.saveConfig();
      } else {
        backupConfig();
      }
    }

    InstallConfiguration.getInstallConfiguration().clearProgressMonitor();
    MMSToolkit.exitToWelcomeGUI(MMSToolkit.getGui());;
  }

}
