/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms;
 * Object:     GUIConfigurableRedoHandler.java
 *
 *************************************************************************/
package gov.nih.nlm.mms;

import java.util.Properties;

/**
 * Handles redoing regular actions in the GUI.
 */

public class GUIConfigurableRedoHandler implements RedoHandler{

  //
  // Fields
  //
  private GUIConfigurable gc = null;
  private MetamorphoSysGUI gui = null;

  /**
   * Instantiates a {@link GUIConfigurableRedoHandler}.
   * @param gc the {@link GUIConfigurable}
   * @param gui the {@link MetamorphoSysGUI}
   */
  public GUIConfigurableRedoHandler(GUIConfigurable gc, MetamorphoSysGUI gui) {
    this.gc = gc;
    this.gui = gui;
  }

  //
  // Methods
  //

  /**
   * Returns the title for the redo menu.
   * @return the title for the redo menu
   */
  public String getTitle() {
    if (gc instanceof MetamorphoSysInputStream)
      return "Redo Input Options action";
    else if (gc instanceof MetamorphoSysOutputStream)
      return "Redo Output Options action";
    return "Redo " + gc.getName() + " action";
  }


  /**
   * Performs the redo operation.
   * @param o1 the previous properties
   * @param o2 the "has data changed" state
   */
  public void redo(Object o1, Object o2) {
    Properties undo_tp =
      (Properties)gc.getProperties().clone();
    Properties p = (Properties)o1;
    Boolean hdc = (Boolean)o2;

    //
    // Sets the configurable's properties
    //
    gc.setProperties(p);
    gc.setDataChanged(hdc.booleanValue());
    gc.setPanelSelections();

    //
    // Bring tab to foreground
    //
    gui.selectTabForFilter(gc);

    //
    // Push onto undo stack
    //
    Object [] undo_tuple = {new GUIConfigurableUndoHandler(gc, gui), undo_tp, hdc};
    gui.pushOnUndoStack(undo_tuple);

  }

}
