/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms;
 * Object:     GUIConfigurableUndoHandler.java
 *
 *************************************************************************/
package gov.nih.nlm.mms;

import java.util.Properties;

/**
 * Handles undoing regular actions in the GUI.
 **/

public class GUIConfigurableUndoHandler implements UndoHandler{

  //
  // Fields
  //
  private GUIConfigurable gc = null;
  private MetamorphoSysGUI gui = null;

  /**
   * Instantiates a {@link GUIConfigurableUndoHandler}.
   * @param gc the {@link GUIConfigurable}
   * @param gui the {@link MetamorphoSysGUI}
   */
  public GUIConfigurableUndoHandler(GUIConfigurable gc, MetamorphoSysGUI gui) {
    this.gc = gc;
    this.gui = gui;
  }

  //
  // Methods
  //

  /**
   * Returns the title for the undo menu.
   * @return the title for the undo menu
   */
  public String getTitle() {
    if (gc instanceof MetamorphoSysInputStream)
      return "Undo Input Options action";
    else if (gc instanceof MetamorphoSysOutputStream)
      return "Undo Output Options action";
    return "Undo " + gc.getName() + " action";
  }


  /**
   * Performs the undo operation.
   * @param o1 the previous properties
   * @param o2 the "has data changed" state
   */
  public void undo(Object o1, Object o2) {
    Properties redo_tp =
      (Properties)gc.getProperties().clone();
    Properties p = (Properties)o1;
    Boolean hdc = (Boolean)o2;

    //
    // Sets the configurable's properties
    //
    gc.setProperties(p);
    gc.setDataChanged(hdc.booleanValue());
    gc.setPanelSelections();

    //
    // Bring tab to foreground
    //
    gui.selectTabForFilter(gc);

    //
    // Push onto redo stack
    //
    Object [] redo_tuple = {new GUIConfigurableRedoHandler(gc, gui), redo_tp, hdc};
    gui.pushOnRedoStack(redo_tuple);

  }

}

