/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms;
 * Object:     InputStreamRedoHandler.java
 *
 *************************************************************************/
package gov.nih.nlm.mms;

import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Handles redo operation for a change in input handler.
 */
public class InputStreamRedoHandler implements RedoHandler{

  //
  // Fields
  //
  private MetamorphoSysGUI gui = null;
  private JTextField field = null;
  private JPanel content_panel = null;

  /**
   * Instantiates an {@link InputStreamRedoHandler}.
   * @param gui the {@link MetamorphoSysGUI}
   * @param field the text field to update
   * @param content_panel the panel to update
   */
  public InputStreamRedoHandler(MetamorphoSysGUI gui,
                                JTextField field,
                                JPanel content_panel) {
    this.gui = gui;
    this.field = field;
    this.content_panel = content_panel;
  }

  //
  // Methods
  //

  /**
   * Returns the title for the redo menu.
   * @return the title for the redo menu
   */
  public String getTitle() {
    return "Redo change input format";
  }

  /**
   * Performs the redo operation.
   * @param o1 the new input stream
   * @param o2 the old input stream
   */
  public void redo(Object o1, Object o2) {
    //
    // Change input stream back
    //
    gui.changeInputStream(((MetamorphoSysInputStream)o1),
                          field, content_panel);

    gui.selectTabForFilter((MetamorphoSysInputStream)o1);

    //
    // Push onto undo stack
    //
    Object[] tuple =
        {new InputStreamUndoHandler(gui,
                                   field, content_panel),
         o1, o2};
     gui.pushOnUndoStack(tuple);
  }

}

