/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms;
 * Object:     InputStreamUndoHandler.java
 *
 *************************************************************************/
package gov.nih.nlm.mms;

import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Handles undo operation for a change in input handler.
 */
public class InputStreamUndoHandler implements UndoHandler{

  //
  // Fields
  //
  private MetamorphoSysGUI gui = null;
  private JTextField field = null;
  private JPanel content_panel = null;

  /**
   * Instantiates an {@link InputStreamUndoHandler}.
   * @param gui the {@link MetamorphoSysGUI}
   * @param field the text field to update
   * @param content_panel the panel to update
   */
  public InputStreamUndoHandler(MetamorphoSysGUI gui,
                                JTextField field,
                                JPanel content_panel) {
    this.gui = gui;
    this.field = field;
    this.content_panel = content_panel;
  }

  //
  // Methods
  //

  /**
   * Returns the title for the undo menu.
   * @return the title for the undo menu
   */
  public String getTitle() {
    return "Undo change input format";
  }

  /**
   * Performs the undo operation.
   * @param o1 the old input stream
   * @param o2 the new input stream
   */
  public void undo(Object o1, Object o2) {
    //
    // Change input stream
    //
    gui.changeInputStream(((MetamorphoSysInputStream)o1),
                          field, content_panel);

    gui.selectTabForFilter((MetamorphoSysInputStream)o1);

    //
    // Prepare redo stack
    //
    Object[] tuple =
        {new InputStreamRedoHandler(gui,
                                   field, content_panel),
         o2, o1};
     gui.pushOnRedoStack(tuple);

  }

}

