/**************************************************************************
 *
 * Package:	gov.nih.nlm.mms
 * Object:	InstallationProgressMonitor.java
 *
 **************************************************************************/
package gov.nih.nlm.mms;

import gov.nih.nlm.swing.FontSizeManager;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.*;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

/**
 * GUI for displaying subsetting progress. Includes installation steps.
 * 
 * @author Deborah Shapiro, Brian Carlsen
 */
public class InstallationProgressMonitor extends MMSProgressMonitor {

  public static int INSTALL_LEXICON = 1;

  public static int INSTALL_NETWORK = 2;
  public static int INSTALL_LVG = 3;

  protected JCheckBox jinstall_lexicon =
    new JCheckBox("Installing Specialist Lexicon & Tools");
  protected JCheckBox jinstall_lvg = 
  	new JCheckBox("Installing LVG");
  protected JCheckBox jinstall_network =
    new JCheckBox("Installing Semantic Network");

  //
  // Constructors
  //
  /**
   * Instantiates an {@link InstallationProgressMonitor} and initializes it.
   */
  public InstallationProgressMonitor() {
    super();
    setTitle("MetamorphoSys Progress..");
    initialize();
  };

  /**
   * Configure the GUI.
   */
  protected void initialize() {
    InstallConfiguration ic = InstallConfiguration.getInstallConfiguration();

    // Configure Layout
    getContentPane().setLayout(new BorderLayout(20, 20));
    JPanel operation_panel = new JPanel(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.fill = GridBagConstraints.BOTH;
    constraints.insets = new Insets(2, 2, 2, 2);

    // Configure Panel
    constraints.gridy = 0;
    constraints.gridwidth = 3;
    constraints.gridy++;
    JLabel operation_label = new JLabel("<html><center>Install UMLS<br>"
        + "The following operations will be performed:<br></center></html>");
    operation_panel.add(operation_label, constraints);
    constraints.gridy++;

    if (ic.isMetathesaurusChecked()) {
      // Remove mouse listeners from checkbox
      jconfig_subset.setEnabled(true);
      MouseListener[] mls = (MouseListener[]) jconfig_subset
          .getListeners(MouseListener.class);
      for (int i = 0; i < mls.length; i++) {
        jconfig_subset.removeMouseListener(mls[i]);
      }
      operation_panel.add(jconfig_subset, constraints);
      constraints.gridy++;
    }

    if (ic.isLexiconChecked()) {
      // Remove mouse listeners from checkbox
      jinstall_lexicon.setEnabled(true);
      MouseListener[] mls = (MouseListener[]) jinstall_lexicon
          .getListeners(MouseListener.class);
      for (int i = 0; i < mls.length; i++) {
        jinstall_lexicon.removeMouseListener(mls[i]);
      }
      operation_panel.add(jinstall_lexicon, constraints);
      constraints.gridy++;
    }
    
    if (ic.isLvgChecked()) {
      // Remove mouse listeners from checkbox
      jinstall_lvg.setEnabled(true);
      MouseListener[] mls =
          (MouseListener[]) jinstall_lvg.getListeners(MouseListener.class);
      for (int i = 0; i < mls.length; i++) {
        jinstall_lvg.removeMouseListener(mls[i]);
      }
      operation_panel.add(jinstall_lvg, constraints);
      constraints.gridy++;
    }
    
    if (ic.isSemanticNetworkChecked()) {
      // Remove mouse listeners from checkbox
      jinstall_network.setEnabled(true);
      MouseListener[] mls = (MouseListener[]) jinstall_network
          .getListeners(MouseListener.class);
      for (int i = 0; i < mls.length; i++) {
        jinstall_network.removeMouseListener(mls[i]);
      }
      operation_panel.add(jinstall_network, constraints);
      constraints.gridy++;
    }

    if (ic.isMetathesaurusChecked()) {
      // Remove mouse listeners from checkbox
      jinstall_meta.setEnabled(true);
      MouseListener[] mls = (MouseListener[]) jinstall_meta
          .getListeners(MouseListener.class);
      for (int i = 0; i < mls.length; i++) {
        jinstall_meta.removeMouseListener(mls[i]);
      }
      operation_panel.add(jinstall_meta, constraints);
      constraints.gridy++;

      // indent
      constraints.insets = new Insets(2, 15, 2, 2);

      // Remove mouse listeners from checkbox
      jinit_cui.setEnabled(true);
      mls = (MouseListener[]) jinit_cui.getListeners(MouseListener.class);
      for (int i = 0; i < mls.length; i++) {
        jinit_cui.removeMouseListener(mls[i]);
      }
      operation_panel.add(jinit_cui, constraints);
      constraints.gridy++;

      // Remove mouse listeners from checkbox
      jsubset_mr.setEnabled(true);
      mls = (MouseListener[]) jsubset_mr.getListeners(MouseListener.class);
      for (int i = 0; i < mls.length; i++) {
        jsubset_mr.removeMouseListener(mls[i]);
      }
      operation_panel.add(jsubset_mr, constraints);
      constraints.gridy++;

      // Remove mouse listeners from checkbox
      jsubset_index.setEnabled(true);
      mls = (MouseListener[]) jsubset_index.getListeners(MouseListener.class);
      for (int i = 0; i < mls.length; i++) {
        jsubset_index.removeMouseListener(mls[i]);
      }
      operation_panel.add(jsubset_index, constraints);
      constraints.gridy++;

      // Remove mouse listeners from checkbox
      jfinal_process.setEnabled(true);
      mls = (MouseListener[]) jfinal_process.getListeners(MouseListener.class);
      for (int i = 0; i < mls.length; i++) {
        jfinal_process.removeMouseListener(mls[i]);
      }
      operation_panel.add(jfinal_process, constraints);
      constraints.gridy++;
    }

    constraints.insets = new Insets(2, 2, 2, 2);
    // Remove mouse listeners from checkbox
    jdone.setEnabled(true);
    MouseListener[] mls = (MouseListener[]) jdone
        .getListeners(MouseListener.class);
    for (int i = 0; i < mls.length; i++) {
      jdone.removeMouseListener(mls[i]);
    }
    operation_panel.add(jdone, constraints);
    constraints.gridy++;

    progress_label.setText("Operation is % Complete");
    operation_panel.add(progress_label, constraints);
    constraints.gridy++;
    // jprogress_bar.setPreferredSize(operation_label.getPreferredSize());
    operation_panel.add(jprogress_bar, constraints);
    constraints.gridy++;

    // Register keystroke
    operation_panel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
        KeyStroke.getKeyStroke("ESCAPE"), cancelAction.getValue(Action.NAME));
    // Register action
    operation_panel.getActionMap().put(cancelAction.getValue(Action.NAME),
        cancelAction);
    operation_panel.add(jcancel, constraints);

    // Initially invisible
    setProgressBarVisible(false);

    // Compose Panels, pack and make visible
    TitledBorder titledBorder = new TitledBorder(new EtchedBorder(
        EtchedBorder.RAISED), "");
    operation_panel.setBorder(new CompoundBorder(titledBorder, new EmptyBorder(
        10, 10, 10, 10)));
    getContentPane().add(operation_panel, BorderLayout.CENTER);
    FontSizeManager.adjustFontSize(this);
    this.pack();
    this.setVisible(true);

    // Configure window listener
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        confirmExit();
      }
    });
  };

  //
  // Methods
  //
  /**
   * Set the indicated operation and repack.
   * 
   * @param oper
   *          the currently running operation
   */
  public void setOperation(int oper) {
    if (oper == CONFIG_SUBSET) {
      jconfig_subset.setSelected(true);
      jinstall_lexicon.setSelected(false);
      jinstall_network.setSelected(false);
      jinstall_meta.setSelected(false);
      jinit_cui.setSelected(false);
      jsubset_mr.setSelected(false);
      jsubset_index.setSelected(false);
      jfinal_process.setSelected(false);
      jdone.setSelected(false);
      // jinstall_lexicon.setEnabled(false);
      // jinstall_network.setEnabled(false);
      setNote("");
      setProgressBarVisible(false);
    } else if (oper == INSTALL_LEXICON) {
      jinstall_lexicon.setSelected(true);
      jinstall_network.setSelected(false);
      jinstall_meta.setSelected(false);
      jinit_cui.setSelected(false);
      jsubset_mr.setSelected(false);
      jsubset_index.setSelected(false);
      jfinal_process.setSelected(false);
      jdone.setSelected(false);
      jconfig_subset.setEnabled(false);
      setNote("");
      setProgressBarVisible(false);
    } else if (oper == INSTALL_NETWORK) {
      jinstall_network.setSelected(true);
      jinstall_meta.setSelected(false);
      jinit_cui.setSelected(false);
      jsubset_mr.setSelected(false);
      jsubset_index.setSelected(false);
      jfinal_process.setSelected(false);
      jdone.setSelected(false);
      jinstall_lexicon.setEnabled(false);
      jconfig_subset.setEnabled(false);
      setNote("");
      setProgressBarVisible(false);
    } else if (oper == INSTALL_META) {
      jinstall_meta.setSelected(true);
      jinit_cui.setSelected(false);
      jsubset_mr.setSelected(false);
      jsubset_index.setSelected(false);
      jfinal_process.setSelected(false);
      jdone.setSelected(false);
      jinstall_lexicon.setEnabled(false);
      jinstall_network.setEnabled(false);
      setNote("");
      setProgressBarVisible(false);
    } else if (oper == INIT_CUIS) {
      jinstall_meta.setSelected(true);
      jinit_cui.setSelected(true);
      jsubset_mr.setSelected(false);
      jsubset_index.setSelected(false);
      jfinal_process.setSelected(false);
      jdone.setSelected(false);
      jconfig_subset.setEnabled(false);
    } else if (oper == SUBSET_META) {
      jinstall_meta.setSelected(true);
      jsubset_mr.setSelected(true);
      jsubset_index.setSelected(false);
      jfinal_process.setSelected(false);
      jdone.setSelected(false);
      jinit_cui.setEnabled(false);
    } else if (oper == SUBSET_INDEXES) {
      jinstall_meta.setSelected(true);
      jsubset_index.setSelected(true);
      jfinal_process.setSelected(false);
      jdone.setSelected(false);
      jsubset_mr.setEnabled(false);
    } else if (oper == FINAL_PROCESSES) {
      jinstall_meta.setSelected(true);
      jfinal_process.setSelected(true);
      jdone.setSelected(false);
      jsubset_index.setEnabled(false);
    } else if (oper == DONE) {
      jdone.setSelected(true);
      jinstall_lexicon.setEnabled(false);
      jinstall_network.setEnabled(false);
      jinstall_meta.setEnabled(false);
      jfinal_process.setEnabled(false);
    }
    this.pack();
  }

}; // End of InstallationProgressMonitor class

