/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms.browser;
 * Class:      AbstractConceptView.java
 *
 *************************************************************************/
package gov.nih.nlm.mms.browser;

import gov.nih.nlm.mms.ApplicationConfiguration;
import gov.nih.nlm.mms.GUIConfigurable;
import gov.nih.nlm.mms.UserConfiguration;
import gov.nih.nlm.swing.DataChangeListener;

import java.awt.Color;
import java.io.File;
import java.util.Properties;

import javax.swing.JEditorPane;
import javax.swing.JPanel;

/**
 * This is an abstract implementation of the {@link ConceptView} interface.
 *
 * @author Deborah Shapiro
 */
public abstract class AbstractConceptView  implements Comparable, ConceptView, FullyPrintable {

  //
  // concept list manager
  //
  protected ConceptListManager clm;

  //
  // Configuration objects
  //
  protected UserConfiguration user_configuration;
  protected ApplicationConfiguration default_configuration;

  /**
   * Indicates whether or not filter data has changed
   * (and therefore needs to be saved).
   */
  protected boolean has_data_changed = false;
  
  protected int font_point_size;
  protected String font_family;

  /**
   * Handles opening of a source directory.
   * @param source_dir the source directory
   */
  public void subsetDirectoryOpened(File source_dir){ }

  /**
   * Sets the {@link ConceptListManager}.
   * @param clm {@link ConceptListManager}
   */
  public void setConceptListManager(ConceptListManager clm) {
    this.clm = clm;
  }

  /**
   * Method provides the filter with values from the default configuration
   * file.
   * @param c {@link ApplicationConfiguration}
   */
  public void setConfiguration(ApplicationConfiguration c) {
    default_configuration = c;
  }

  /**
   * Method provides the filter with values from the user configuration
   * file.
   * @param c {@link UserConfiguration}
   */
  public void setConfiguration(UserConfiguration c) {
    user_configuration = c;
  }

  /**
   * Method returns a {@link String} detailing
   * the user selections on the filter.  This {@link String} will
   * be displayed in the mmsys.log at the completion of subsetting
   * so it should be formatted to be easily read.
   * @return {@link String} of data for log
   */
  public String getConfigurationDetails() {
    return "config details";
  }

  /**
   * Returns the {@link Properties} representing the configuration choices.
   * @return the {@link Properties} representing the configuration choices
   */
  public Properties getProperties() {
    return null;
  }

  /**
   * Sets the {@link Properties} representing the configuration choices.
   * @param p the {@link Properties} representing the configuration choices
   */
  public void setProperties(Properties p) { }

  /**
   * Indicates whether or not the configuration information has changed.
   * @return <code>true</code> if data has changed;
   *         <code>false</code> otherwise
   */
  public boolean hasDataChanged() {
    return has_data_changed;
  }

  /**
   * Sets the value of the data changed flag.
   * @param b the <code>boolean</code> value for the data changed flag
   */
  public void setDataChanged(boolean b) {
    has_data_changed = b;
  }

  /**
   * Sets the data changed flag to <code>false</code>.
   */
  public void resetDataChanged() {
    has_data_changed = false;
  }

  /**
   * Adds a {@link DataChangeListener} that responds to configuration changes.
   * Allows main GUI to record all changes to
   * this gui in the undo/redo mechanism.
   * @param dcl the {@link DataChangeListener} to add
   */
  public void addDataChangeListener(DataChangeListener dcl) { }

  /**
   * Removes the specified {@link DataChangeListener}.
   * @param dcl the {@link DataChangeListener} to remove
   */
  public void removeDataChangeListener(DataChangeListener dcl) {  }

  /**
   * Returns the {@link Color} that should be used to display
   * the name.
   * @return the {@link Color} that should be used to display the name.
   */
  public Color getColor() {
    return null;
  }

  /**
   * Returns the description to be used in conjunction with the panel
   * when presented to the user.
   * @return the description
   */
  public String getTabDescription() {
    return "description";
  }

  /**
   * Returns a {@link JPanel} that will be displayed when the
   * user selects the corresponding menu item from the Help menu
   * and will provide the user with detailed instructions on the tab's use.
   * @return the {@link JPanel} to be used when showing user help
   */
  public JPanel getHelpPanel() {
    return null;
  }

  /**
   * Updates the view to reflect the data model.
   */
  public void setPanelSelections() {  }

  /**
   * Indicates whether or not the validation rules have passed.
   * @return <code>boolean</code><code>true</code> if so;
   * 	     <code>false</code> otherwise
   */
  public boolean validate() {
    return true;
  }

  /**
   * Returns a message indicating why validation failed.  Should only be
   * called if {@link #validate()} returns <code>false</code>.
   * @return the message
   */
  public String getValidateMessage() {
    return "";
  }

  /**
   * Returns the messages indicating why validation failed.  Should only be
   * called if {@link #validate()} returns <code>false</code>.
   * @return the failure messages
   */
  public String[] getValidationErrors() {
    return new String [] {getValidateMessage()};
  }

  /**
   * Returns warnings that were indicated during validation.
   * @return the warning messages
   */
  public String[] getValidationWarnings() {
    return new String [0];
  }

  /**
   * Returns the {@link GUIConfigurable} whose tab should next be displayed
   * to allow the user to correct an error or warning.
   * @return GUIConfigurable to be displayed
   */
  public GUIConfigurable getGUIConfigurableForValidationFailure() {
    return null;
  }

  /**
   * Compares this object with the specified object for order.  Returns a
   * negative integer, zero, or a positive integer as this object is less
   * than, equal to, or greater than the specified object.<p>
   * <p/>
   * Required method for implementing {@link Comparable} interface.
   * Indicates the means for comparison between two {@link ConceptView}s.
   * @param o the Object to be compared.
   * @return a negative integer, zero, or a positive integer as this object
   *         is less than, equal to, or greater than the specified object.
   * @throws ClassCastException if the specified object's type is not
   *         {@link GUIConfigurable}.
   */
  public int compareTo(Object o) {
    if (!(o instanceof GUIConfigurable)) {
      throw new ClassCastException();
    }
    return getName().compareTo(((GUIConfigurable) o).getName());
  }

  /**
   * Searches the text displayed in the {@link JEditorPane}
   * for the given search_term.  Starts the search at the given
   * start position within the text.
   */
  public void find() { }
  
  /**
   * Sets the font point size for this {@link ConceptView}.
   * @param point_size <code>int</code>
   */
  public void setFontPointSize(int point_size) {
  	this.font_point_size = point_size;
  }
  
  /**
   * Sets the font family for this {@link ConceptView}.
   * @param font_family <code>int</code>
   */
  public void setFontFamily(String font_family) {
  	this.font_family = font_family;
  }

}

