/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms.browser;
 * Class:      CodeSearchFinder.java
 *
 *************************************************************************/
package gov.nih.nlm.mms.browser;

import gov.nih.nlm.mms.MMSToolkit;
import gov.nih.nlm.mms.filters.SourcesToRemoveFilter;
import gov.nih.nlm.swing.SuperJList;
import gov.nih.nlm.swing.SwingToolkit;
import gov.nih.nlm.umls.io.RichMRFilesIndexLookup;
import gov.nih.nlm.umls.meta.Atom;
import gov.nih.nlm.umls.meta.Concept;
import gov.nih.nlm.umls.meta.Source;
import gov.nih.nlm.umls.meta.impl.ConceptImpl;
import gov.nih.nlm.util.FieldedStringTokenizer;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;



/**
 * Represents a {@link Finder} that directly looks up concepts by CODE values.
 * 
 * @author Deborah Shapiro
 */
public class CodeSearchFinder extends AbstractFinder {

  //
  // Search field
  //
  private JTextField search_field;

  //
  // search button
  //
  private JButton go_button;
  
  //
  // Full list of word search concept results
  //
  private Concept[] concept_candidates;
  
  //
  // Source(s) from which to display Atoms with matching codes 
  //  
  private Set selected_sources = new HashSet();
  
  //
  // Gui Elements
  //
  private JButton next_button;

  private JButton prev_button;
  
  private JList cui_choice_list;

  private JScrollPane cui_list_sp;
  
  //
  // results label
  //
  private JLabel list_label; 
  
  //
  // index of highest ranked concept currently displayed
  //
  private int highest_displayed_concept = 0;

  //
  // index of lowest ranked concept currently displayed
  //
  private int first_displayed = 0;  

  
  /**
   * Instantiates a {@link CodeSearchFinder}.
   */
  public CodeSearchFinder() {
  }

  /**
   * Returns a {@link JPanel} containing configuration options.
   * 
   * @return a {@link JPanel} containing configuration options
   */
  public JPanel getPanel() {

    // top level panel
    JPanel code_search_panel = new JPanel();
    code_search_panel.setBorder(new EmptyBorder(5, 5, 5, 5));
    code_search_panel.setLayout(new BorderLayout());

    // panel with label, field, button
    JPanel button_panel = new JPanel();
    BorderLayout bl = new BorderLayout(5, 5);
    button_panel.setLayout(bl);
    JLabel search_label = new JLabel("Enter a CODE:");
    button_panel.add(search_label, BorderLayout.NORTH);
    search_field = new JTextField(8);
    search_field.addKeyListener(new KeyAdapter() {
      public void keyTyped(KeyEvent e) {
        char c = e.getKeyChar();
        // hitting ENTER causes search
        if (c == KeyEvent.VK_ENTER) {
          CodeSearchAction csa = new CodeSearchAction();
          csa.actionPerformed(new ActionEvent(this, 0, ""));
        }
      }
    });
    button_panel.add(search_field, BorderLayout.CENTER);
    go_button = new JButton(new CodeSearchAction());

    button_panel.add(go_button, BorderLayout.SOUTH);
    code_search_panel.add(button_panel, BorderLayout.NORTH);
    
    // initialize cui choice list
    final JPanel cui_choice_panel = new JPanel(new BorderLayout(5, 5));
    final JPanel label_panel = new JPanel(new BorderLayout(5, 5));
    final JPanel prev_next_panel = new JPanel(new FlowLayout());
    list_label = new JLabel("Select a concept.");
    label_panel.add(list_label, BorderLayout.WEST);
    next_button = new JButton(new ForwardAction());
    if (next_button.getIcon() != null) {
      next_button.setText(""); // an icon-only button
    }
    next_button.setMargin(new Insets(2, 2, 2, 2));

    prev_button = new JButton(new BackwardAction());
    if (prev_button.getIcon() != null) {
      prev_button.setText(""); // an icon-only button
    }
    prev_button.setMargin(new Insets(2, 2, 2, 2));

    prev_next_panel.add(prev_button);
    prev_next_panel.add(next_button);
    label_panel.add(prev_next_panel, BorderLayout.EAST);
    cui_choice_panel.add(label_panel, BorderLayout.NORTH);
    cui_choice_list = new SuperJList(new String[] {});
    cui_choice_list.addMouseListener(new MouseAdapter() {

      public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
          Thread t = new Thread(new Runnable() {
            public void run() {
            	if (concept_candidates == null || 
            			concept_candidates.length == 0)
            		return;
              clm.addConcept( (Concept) cui_choice_list.getSelectedValue());
              RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
            }
          });
          t.start();

        }
      }
    });

    cui_list_sp = new JScrollPane(cui_choice_list,
        ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    cui_choice_panel.add(cui_list_sp, BorderLayout.CENTER);


    code_search_panel.add(cui_choice_panel, BorderLayout.CENTER);
    
    
    //
    //Create the popup menu.
    //
    JPopupMenu popup = new JPopupMenu();
    JMenuItem option_menu_item = new JMenuItem("Options...");
    option_menu_item.addActionListener( 
    	new ActionListener() {
    	  public void actionPerformed(ActionEvent e) {
    	    JMenuItem source = (JMenuItem)(e.getSource());
    	    String s = source.getText();
    	    if (s.equals("Options...")) {
    	      showOptions();
    	    }
    	  }
    	}
    );
    popup.add(option_menu_item);

    //Add listener to components that can bring up popup menus.
    PopupListener popup_listener = new PopupListener(popup);
    code_search_panel.addMouseListener(popup_listener);

    return code_search_panel;    
  }
  
  /**
   * Handles opening of a source directory.
   * 
   * @param source_dir
   *          the source directory
   */
  public void subsetDirectoryOpened(File source_dir) {
		if (next_button != null)
			next_button.setEnabled(false);
		if (prev_button != null)
			prev_button.setEnabled(false);
  }

  /**
   * Method returns all properties specific to the {@link ConceptView}.
   * 
   * @return {@link Properties} only those directly related to the concept view.
   */
  public Properties getProperties() {
    Properties new_filter_props = new Properties();

    List sources = new ArrayList(selected_sources);
    StringBuffer line = new StringBuffer();
    for (int i = 0; i < sources.size(); i++) {
      line.append(((Source) sources.get(i)).getRootSourceAbbreviation());
      line.append("|");
      line.append(((Source) sources.get(i)).getSourceFamily());
      if (i <= sources.size() - 2)
        line.append(";");
    }
    new_filter_props.setProperty("selected_sources", line.toString());
	return new_filter_props;
  }

  /**
   * Sets the view properties.
   * @param p {@link Properties}
   */
  public void setProperties(Properties p) {
    Properties filter_props = p;

    if (filter_props.containsKey("selected_sources")) {
      selected_sources.clear();
      String remove_prop = (String) filter_props.get("selected_sources");
      String[] tokens = FieldedStringTokenizer.split(remove_prop, ";");
      for (int i = 0; i < tokens.length; i++) {
        Source local_source = Source.getSource(tokens[i].substring(0, tokens[i]
            .indexOf("|")));
        if (local_source != null)
          selected_sources.add(local_source);
      }
    }
  }  
    
  /**
   * Resets the configuration back to default values. When used with a GUI this
   * method should change the data model behind the GUI and allow
   * <code>setPanelSelections</code> to actually update the GUI.
   */
  public void resetToDefaults() {
    search_field.setText("");
    cui_choice_list.setListData(new Object[0]);
    list_label.setText("Select a concept.");
  }

  /**
   * Highlights the text displayed in the word text field.
   */
  public void highlightText() {
    search_field.requestFocus();
    search_field.setSelectionStart(0);
    search_field.setSelectionEnd(search_field.getText().length());
    cui_choice_list.setListData(new Object[0]);
    list_label.setText("Select a concept.");

  }

  /**
   * Returns the name.
   * 
   * @return the name
   */
  public String getName() {
    return "Code Search";
  }

  /**
   * Presents an advanced options GUI to the user.
   */
  public void showOptions() {
	  final CodeSearchFinderOptionsGUI code_search_finder_options = 
		  new CodeSearchFinderOptionsGUI(this);
      code_search_finder_options.setVisible(true);
  }

  /**
   * Indicates whether or not an options menu item should be supplied for
   * advanced configuration options.
   * 
   * @return <code>true</code> if so; <code>false</code> otherwise
   */
  public boolean hasAdvancedOptions() {
    return true;
  }

  /**
   * Redraws the text portions of the {@link Finder}.
   */
  public void refresh() {
  	search_field.setFont(new Font(font_family, Font.PLAIN, font_point_size));
    cui_choice_list.setFont(new Font(font_family, Font.PLAIN, font_point_size));

    int row_ct = 100;
    Concept[] this_page_list = new Concept[row_ct];
    int index = 0;
    highest_displayed_concept = 0;
		if (concept_candidates == null)
			return;
		for (int i = highest_displayed_concept; i < highest_displayed_concept
				+ row_ct
				&& i < concept_candidates.length; i++)
			this_page_list[index++] = concept_candidates[i];
		
		list_label.setText("Select a concept. (" + (highest_displayed_concept + 1)
				+ " to " + (highest_displayed_concept + index) + " of "
				+ concept_candidates.length + " results)");
		first_displayed = highest_displayed_concept + 1;
		highest_displayed_concept += index;

    cui_choice_list.setListData(this_page_list);

    prev_button.setEnabled(false);
    next_button.setEnabled(highest_displayed_concept < concept_candidates.length);
  }  
  
  //
  // Inner classes
  //
  public class PopupListener extends MouseAdapter {
    JPopupMenu popup;

    PopupListener(JPopupMenu popupMenu) {
      popup = popupMenu;
    }

    public void mousePressed(MouseEvent e) {
      maybeShowPopup(e);
    }

    public void mouseReleased(MouseEvent e) {
      maybeShowPopup(e);
    }

    private void maybeShowPopup(MouseEvent e) {
      if (e.isPopupTrigger()) {
        popup.show(e.getComponent(),
                   e.getX(), e.getY());
      }
    }
  }
  
  /**
   * Action for the "Search" button.
   */
  private class CodeSearchAction extends AbstractAction {
	  
	  /**
	   * Instantiates a {@link CodeSearchAction}.
	   */
	  public CodeSearchAction() {
		  super();
		  putValue(Action.NAME, "Get Concepts..");
		  putValue(Action.SHORT_DESCRIPTION, "Code Lookup");
		  putValue(Action.LONG_DESCRIPTION, "Code Lookup");
	  }	  
	  
	  /**
	   * @param e
	   *          an {@link ActionEvent}
	   */
	  public void actionPerformed(ActionEvent e) {
		  RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
		  Thread t = new Thread(new Runnable() {
			  public void run() {
				  try {
					  String code = search_field.getText().toUpperCase();
					  //
					  // Validate search field
					  //
					  if (code.equals("")) {
						  MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
								  "A code must be entered.", false);
						  RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
						  return;
					  }
										  
					  //
					  // Configure lookup object
					  //
					  final RichMRFilesIndexLookup index_lookup = new RichMRFilesIndexLookup(
							  user_configuration.getSourcePaths()[0]);
					 
					  Atom[] matched_atoms = index_lookup.findAtomsByCode(code);
					  if (selected_sources.size() > 0) {
					    List remaining_atoms = new ArrayList();
					    for (int i=0; i<matched_atoms.length; i++) {
					      if (selected_sources.contains(matched_atoms[i].getSource()))
					    	remaining_atoms.add(matched_atoms[i]);
					    }
					    matched_atoms = (Atom[])remaining_atoms.toArray(new Atom[0]);
					  }
					  List matched_concepts = new ArrayList();
					  for (int i =0; i<matched_atoms.length; i++) {
					    Concept cpt = new DisplayableConcept(matched_atoms[i].getConcept(),0);
					    if (!matched_concepts.contains(cpt))
					      matched_concepts.add(cpt);
					    else {
					      int index = matched_concepts.indexOf(cpt);
					      Concept prior_cpt = (Concept)matched_concepts.get(index);
					      prior_cpt.addAtom(matched_atoms[i]);
					    }			      
					  }
					  
					  concept_candidates = (Concept[])matched_concepts.toArray(new Concept[0]);
					  Arrays.sort(concept_candidates);		  
					  index_lookup.close();
					 	  
					  
					  if (concept_candidates.length == 0) {
						  highlightText();
						  MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
								  "No concepts matched the "
								  + " search code.", false);
						  RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
						  return;
					  }
					  
					  final int row_ct = 100;
					  String text = "Select a concept. (";
					  if (concept_candidates.length > row_ct) {
						  text += "1 to " + row_ct + " of ";
						  text += (concept_candidates.length);
						  first_displayed = 1;
						  highest_displayed_concept = row_ct;
					  } else {
						  text += concept_candidates.length;
						  first_displayed = 1;
						  highest_displayed_concept = concept_candidates.length;
					  }
					  
					  final String list_label_text;
					  if (concept_candidates.length > 1 || concept_candidates.length == 0)
						  list_label_text = text + " results)";
					  else if (concept_candidates.length == 1) {
						  list_label_text = text + " result)";
						  Thread t = new Thread(new Runnable() {
							  public void run() {
								  clm.addConcept(concept_candidates[0]);
								  RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
							  }
						  });
						  t.start();
					  } else
						  list_label_text = "";
					  
					  SwingUtilities.invokeAndWait(new Runnable() {
						  public void run() {
							  
							  list_label.setText(list_label_text);
							  if (concept_candidates.length > row_ct) {
								  Concept[] first_page_candidates = new Concept[row_ct];
								  for (int i = 0; i < row_ct; i++) {
									  first_page_candidates[i] = concept_candidates[i];
								  }
								  cui_choice_list.setListData(first_page_candidates);
								  next_button.setEnabled(true);
							  } else {
								  cui_choice_list.setListData(concept_candidates);
								  next_button.setEnabled(false);
							  }
							  
							  prev_button.setEnabled(false);
						  }
					  });
					  
				  } catch (Exception e) {
					  e.printStackTrace();
					  MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
							  "Unexpected error handling word search."
							  + "\n See console for more details.", false);
				  }
				  RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
			  }
		  });
		  t.start();
		  
	  } // end actionPerformed
	  
  } // end inner class

  public class ForwardAction extends AbstractAction {

	    /**
	     * Instantiates a {@link ForwardAction}.
	     */
	    public ForwardAction() {
	      super();

	      putValue(Action.NAME, "Forward");
	      putValue(Action.SHORT_DESCRIPTION, "Next page");
	      ImageIcon clm_icon = null;
	      BufferedImage clm_image = null;
	      try {
	        URL gif_url = this.getClass().getClassLoader().getResource(
	            "config/arrows_blue_005.gif");
	        clm_image = SwingToolkit.readScaledImage(gif_url, 17, false);
	        clm_icon = new ImageIcon(clm_image);
	      } catch (Exception ex) {
	        throw new RuntimeException(
	            "Scaled version of config/arrows_blue_005.gif could not be loaded.");
	      }
	      putValue(Action.SMALL_ICON, clm_icon);
	      setEnabled(false);
	    }

	    /**
	     * Action performed.
	     * 
	     * @param e
	     *          {@link ActionEvent}
	     */
	    public void actionPerformed(ActionEvent e) {
	      if (highest_displayed_concept >= concept_candidates.length)
	        return;
	      int row_ct = 100;
	      Concept[] this_page_list = new Concept[row_ct];
	      int index = 0;
	      for (int i = highest_displayed_concept; i < highest_displayed_concept
	          + row_ct
	          && i < concept_candidates.length; i++)
	        this_page_list[index++] = concept_candidates[i];
	      list_label.setText("Select a concept. ("
	          + (highest_displayed_concept + 1) + " to "
	          + (highest_displayed_concept + index) + " of "
	          + concept_candidates.length + " results)");
	      first_displayed = highest_displayed_concept + 1;
	      highest_displayed_concept += index;

	      cui_choice_list.setListData(this_page_list);

	      prev_button.setEnabled(true);
	      next_button
	          .setEnabled(highest_displayed_concept < concept_candidates.length);
	    }
	  }

	  public class BackwardAction extends AbstractAction {

	    /**
	     * Instantiates a {@link BackwardAction}.
	     */
	    public BackwardAction() {
	      super();

	      putValue(Action.NAME, "Previous");
	      putValue(Action.SHORT_DESCRIPTION, "Previous page");
	      ImageIcon clm_icon = null;
	      BufferedImage clm_image = null;
	      try {
	        URL gif_url = this.getClass().getClassLoader().getResource(
	            "config/arrows_blue_004.gif");
	        clm_image = SwingToolkit.readScaledImage(gif_url, 17, false);
	        clm_icon = new ImageIcon(clm_image);
	      } catch (Exception ex) {
	        throw new RuntimeException(
	            "Scaled version of config/arrows_blue_004.gif could not be loaded.");
	      }
	      putValue(Action.SMALL_ICON, clm_icon);

	      setEnabled(false);
	    }

	    /**
	     * Action performed.
	     * 
	     * @param e
	     *          {@link ActionEvent}
	     */
	    public void actionPerformed(ActionEvent e) {
	      int row_ct = 100;
	      if (first_displayed - (row_ct + 1) < 0)
	        return;
	      Concept[] this_page_list = new Concept[row_ct];
	      int index = 0;
	      for (int i = first_displayed - (row_ct + 1); i < first_displayed - 1; i++)
	        this_page_list[index++] = concept_candidates[i];
	      first_displayed -= row_ct;
	      highest_displayed_concept = first_displayed + (row_ct - 1);
	      list_label.setText("Select a concept. (" + first_displayed + " to "
	          + highest_displayed_concept + " of " + concept_candidates.length
	          + " results)");

	      cui_choice_list.setListData(this_page_list);

	      next_button.setEnabled(true);
	      prev_button.setEnabled(first_displayed != 1);
	    }
	  }  
	  
	  /**
	   * Provides the Report View Options GUI.
	   */
	  private class CodeSearchFinderOptionsGUI extends JDialog {

	    //
	    // Private Fields
	    //
	    private SuperJList source_list = new SuperJList();


	    /**
	     * Default constructor.
	     * 
	     * @param f
	     *          {@link ExpandableConceptView}
	     */
	    public CodeSearchFinderOptionsGUI(CodeSearchFinder f) {
	      super(RRFBrowserGUI.getRRFBrowserGUI());
	      setModal(true);
	      setTitle("Code Search Options");
	      initialize();
	    };

	    //
	    // Methods
	    //

	    /**
	     * This method sets up the GUI
	     */
	    private void initialize() {

	      final DataChangeListener dcl = new DataChangeListener();

	      getContentPane().setLayout(new BorderLayout());

	      // configure source_lang_panel
	      JPanel source_lang_panel = new JPanel();
	      BoxLayout source_lang_layout = new BoxLayout(source_lang_panel,
	          BoxLayout.Y_AXIS);
	      source_lang_panel.setLayout(source_lang_layout);

	      JLabel list_label = new JLabel("Select source(s) to be searched.");
	      source_lang_panel.add(list_label);
	      SourcesToRemoveFilter strf = new SourcesToRemoveFilter();
	      strf.setConfiguration(default_configuration);
	      strf.setProperties(user_configuration.getProperties(strf));
	      List sources = (strf.removeSelectedSources() ? strf
	          .getSelectedSourcesInverse() : strf.getSelectedSources());
	      Source[] sorted_sources = (Source[]) sources.toArray(new Source[0]);
	      Arrays.sort(sorted_sources);
	      source_list = new SuperJList(sorted_sources);
	      source_list
	          .setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
	      if (!selected_sources.isEmpty()) {
	        Source[] source_array = (Source[]) selected_sources
	            .toArray(new Source[0]);
	        source_list.setSelectedValues(source_array, false);
	      } else {
	    	source_list.setSelectedValues(sorted_sources, false);
	      }

	      JScrollPane sp = new JScrollPane(source_list,
	          ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
	          ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	      source_lang_panel.add(sp);

	  
	      // configure done_panel
	      FlowLayout done_layout = new FlowLayout();
	      done_layout.setVgap(10);
	      JPanel done_panel = new JPanel(done_layout);
	      JPanel complete_panel = new JPanel(new BorderLayout(10, 10));

	      JButton jdone = new JButton("Done");
	      jdone.addActionListener(dcl);
	      done_panel.add(jdone);

	      // Create an action object
	      Action cancelAction = new AbstractAction("Cancel") {
	        {
	          // Set action command to "Cancel"
	          putValue(Action.ACTION_COMMAND_KEY, getValue(Action.NAME));
	        }

	        // This method is called when the action is invoked
	        public void actionPerformed(ActionEvent e) {
	          dcl.actionPerformed(e);
	        }
	      };
	      JButton jcancel = new JButton(cancelAction);
	      // Register keystroke
	      done_panel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
	          KeyStroke.getKeyStroke("ESCAPE"), cancelAction.getValue(Action.NAME));
	      // Register action
	      done_panel.getActionMap().put(cancelAction.getValue(Action.NAME),
	          cancelAction);
	      done_panel.add(jcancel);

	      // Add the sub-panels to complete_panel
	      complete_panel.add(source_lang_panel, BorderLayout.CENTER);
	      complete_panel.add(done_panel, BorderLayout.SOUTH);
	      EtchedBorder etched_border = new EtchedBorder(EtchedBorder.RAISED);
	      complete_panel.setBorder(new CompoundBorder(etched_border,
	          new EmptyBorder(10, 10, 10, 10)));

	      getContentPane().add(complete_panel, BorderLayout.CENTER);
	      getRootPane().setDefaultButton(jdone);
	      this.pack();
	      this.setLocationRelativeTo(null);
	    };

	    //
	    // Inner classes
	    //
	    public class DataChangeListener implements java.awt.event.ActionListener {

	      // constructor
	      public DataChangeListener() {
	      };

	      // implementation of ActionListener interface
	      public void actionPerformed(ActionEvent e) {

	        if (e.getActionCommand().equals("Done")) {

	          selected_sources.clear();
	          Object[] selected = source_list.getSelectedValues();
	          for (int i = 0; i < selected.length; i++) {
	            selected_sources.add((Source) selected[i]);
	          }

	          //
	          // Source(s) to display (if restricted)
	          //
	            RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();

	            Thread t = new Thread(new Runnable() {
	              public void run() {

	                clm.refresh();
	                RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
	              }
	            });
	            t.start();

	          
	          CodeSearchFinderOptionsGUI.this.dispose();
	        } else if (e.getActionCommand().equals("Cancel"))
	          dispose();
	      }
	    }
	  } // End of CodeSearchFinderOptionsGUI class
	  /**
	   * Extension of {@link Concept} to support a configurable display name.
	   */
	  private class DisplayableConcept extends ConceptImpl {

	    /**
	     * Instantiates a {@link DisplayableConcept} from the specified {@link Concept}
	     * @param concept the {@link Concept}
	     * @param rank the rank of the {@link Concept}
	     */
	    public DisplayableConcept(Concept concept, double rank) {
	      super();
	      setAtoms(concept.getAtoms());
	      setCui(concept.getCui());
	    }

	    /**
	     * Returns the preferred name of the concept.
	     * @return the preferred name of the concept
	     */
	    public String getPreferredName() {
	      Atom[] atoms = getAtoms();
	      Arrays.sort(atoms, new NameComparator());
	      return atoms[0].getStr();
	    }

	    /**
	     * Returns a {@link String} representation of the concept based
	     * on the prefererd name.
	     * @return  a {@link String} representation of the concept
	     */
	    public String toString() {
	      String preferred_string = null;
	      preferred_string = getPreferredName();

	      if (preferred_string != null)
	        return super.getCui() + " " + preferred_string; // + " " + rank;
	      else
	        return super.getCui();
	    }
	  } 
	  
	  //
	  // Comparator used to sort the preferred strings to be used for 
	  // display in the list of Concepts returned.
	  //
	  public class NameComparator implements Comparator {

	    /**
	     * Instantiates a comparator.
	     */
	    public NameComparator() {
	    }

	    //
	    // Implementation of Comparator interface
	    //

	    /**
	     * Compares the specified field of each {@link Atom} object.
	     * 
	     * @param o1
	     *          the first {@link Object} to be compared.
	     * @param o2
	     *          the second {@link Object} to be compared.
	     * @return An <code>int</code> which might be negative integer, zero, or a
	     *         positive integer as the first argument is less than, equal to, or
	     *         greater than the second.
	     */
	    public int compare(Object o1, Object o2) {
	      if (o1 == null || o2 == null || !(o1 instanceof Atom)
	          || !(o2 instanceof Atom))
	        return 0;
	      Atom a1 = (Atom) o1;
	      Atom a2 = (Atom) o2;

	      String lat1 = a1.getLat().getAbbreviation();
	      String lat2 = a2.getLat().getAbbreviation();
	      String ts1 = a1.getTs().getAbbreviation();
	      String ts2 = a2.getTs().getAbbreviation();
	      String stt1 = a1.getStt().getAbbreviation();
	      String stt2 = a2.getStt().getAbbreviation();
	      String ispref1 = a1.getIspref();
	      String ispref2 = a2.getIspref();
	      int len1 = a1.getStr().length();
	      int len2 = a2.getStr().length();

	      if (lat1.equals("ENG") && !lat2.equals("ENG"))
	    	  return -1;
	      else if (!lat1.equals("ENG") && lat2.equals("ENG"))
	      	  return 1;
	      
	      if (ts1.equals("P") && !ts2.equals("P"))
	    	  return -1;
	      else if (!ts1.equals("P") && ts2.equals("P"))
	    	  return 1;
	      
	      if (stt1.equals("PF") && !stt2.equals("PF"))
	    	  return -1;
	      else if (!stt1.equals("PF") && stt2.equals("PF"))
	    	  return 1;
	      
	      if (ispref1.equals("Y") && !ispref2.equals("Y"))
	    	  return -1;
	      else if (!ispref1.equals("Y") && ispref2.equals("Y"))
	    	  return 1;

	      return len2 - len1;

	    }
	  }
	  
}
