/*****************************************************************************
 * Package: gov.nih.nlm.mms.browser
 * Object:  ConceptView.java
 *
 ****************************************************************************/
package gov.nih.nlm.mms.browser;

import gov.nih.nlm.mms.GUIConfigurable;
import gov.nih.nlm.umls.meta.Concept;
import gov.nih.nlm.util.FinderHandler;

import java.io.File;

import javax.swing.JEditorPane;

/**
 * Generically represents a view of a concept.
 *
 * @author Deborah Shapiro
 */
public interface ConceptView extends GUIConfigurable, FullyPrintable {

  /**
   * Sets the {@link ConceptListManager}.
   * @param clm {@link ConceptListManager}
   */
  public void setConceptListManager(ConceptListManager clm);

  /**
   * Sets the {@link Concept} to be displayed.
   * @param concept {@link Concept}
   */
  public void setConcept(Concept concept);

  /**
   * Returns the content type used for the
   * {@link JEditorPane} display.
   * @return content type
   */
  public String getContentType();

  /**
   * Sets the font point size for this {@link ConceptView}.
   * @param point_size <code>int</code>
   */
  public void setFontPointSize(int point_size);
  
  /**
   * Sets the font family for this {@link ConceptView}.
   * @param font_family <code>int</code>
   */
  public void setFontFamily(String font_family);

  /**
   * Indicates the {@link FinderHandler} that will
   * execute the find operation for this {@link ConceptView}
   * @return {@link FinderHandler}
   */
  public FinderHandler getFinderHandler();

  /**
   * Handles opening of a source directory.
   * @param source_dir the source directory
   */
  public void subsetDirectoryOpened(File source_dir);

}
