/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms.browser;
 * Class:      ExpandableRawView.java
 *
 *************************************************************************/
package gov.nih.nlm.mms.browser;

import gov.nih.nlm.mms.MMSToolkit;
import gov.nih.nlm.mms.RichMRMetamorphoSysInputStream;
import gov.nih.nlm.swing.DataChangeListener;
import gov.nih.nlm.umls.io.RichMRFilesConceptOutputStream;
import gov.nih.nlm.umls.io.RichMRFilesIndexLookup;
import gov.nih.nlm.umls.meta.Atom;
import gov.nih.nlm.umls.meta.AtomCoocurrence;
import gov.nih.nlm.umls.meta.AtomMapping;
import gov.nih.nlm.umls.meta.AtomRelationship;
import gov.nih.nlm.umls.meta.Attribute;
import gov.nih.nlm.umls.meta.Concept;
import gov.nih.nlm.umls.meta.ConceptMapping;
import gov.nih.nlm.umls.meta.Context;
import gov.nih.nlm.umls.meta.ContextMember;
import gov.nih.nlm.umls.meta.Coocurrence;
import gov.nih.nlm.umls.meta.Mapping;
import gov.nih.nlm.umls.meta.Relationship;
import gov.nih.nlm.umls.meta.ReleaseMetadata;
import gov.nih.nlm.umls.meta.SourceTermType;
import gov.nih.nlm.umls.meta.impl.ConceptImpl;
import gov.nih.nlm.util.FieldedStringTokenizer;
import gov.nih.nlm.util.FinderHandler;
import gov.nih.nlm.util.HTMLDocumentRenderer;
import gov.nih.nlm.util.OnePagePrintable;
import gov.nih.nlm.util.SystemToolkit;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.swing.*;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

/**
 * {@link ConceptView} that displays the concept report with a formatted
 * appearance that is customizable by the user.
 * 
 * @author Deborah Shapiro
 * 
 */

public class ExpandableRawView extends AbstractConceptView implements
    ActionListener {

  //
  // GUI
  //
  private JEditorPane report_pane;

  //
  // Current cui
  //
  private Concept concept;

  //
  // Prepare string buffer
  //
  private StringBuffer report = new StringBuffer(10000);

  //
  // Mechanism for looking up children and siblings
  //
  private RichMRFilesIndexLookup index_lookup = null;

  //
  // Indicates which nodes have already been expanded
  //
  private Map expanded_hm = null;

  //
  // plus and minus icons for table
  //
  private URL plus_url = null, minus_url = null;

  private static String ui = "";

  //
  // Change file names
  //
  protected String[] change_files;

  //
  // Prepares separators for atom reports
  // and link parameters for HTML reports
  //
  private String bold_begin = null;

  private String bold_end = null;

  final private String line_end = "<br>" + System.getProperty("line.separator");

  private SingleConceptOutputToStringBuffer rrf;

  private Rectangle rect;

  private ReleaseMetadata release_metadata;

  private int page_size = 100;

  /**
   * Instantiates a {@link ExpandableRawView}.
   */
  public ExpandableRawView() {
    expanded_hm = new HashMap();
    try {
      plus_url = this.getClass().getClassLoader().getResource(
          "config/plus2.gif");
      minus_url = this.getClass().getClassLoader().getResource(
          "config/minus2.gif");
    } catch (Exception ex) {
      throw new RuntimeException(
          "config/plus.gif or config/minus.gif could not be loaded.");
    }

  }

  /**
   * Returns a {@link JPanel} containing the concept report.
   * 
   * @return a {@link JPanel} containing the concept report
   */
  public JPanel getPanel() {

    JPanel report_panel = new JPanel();
    report_panel.setLayout(new BoxLayout(report_panel, BoxLayout.Y_AXIS));
    report_pane = new JEditorPane();
    report_pane.addHyperlinkListener(new HyperlinkListener() {
      public void hyperlinkUpdate(HyperlinkEvent he) {
        rect = report_pane.getVisibleRect();
        if (he.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
          ui = he.getDescription();
          if (ui.startsWith("CUI:")) {
            Thread thread = new Thread(new Runnable() {
              public void run() {
                Concept local_concept = new ConceptImpl();
                local_concept.setCui(ui.substring(4));
                clm.addConcept(local_concept);
                RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
              }
            });
            RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
            thread.start();
          } else if (ui.startsWith("AUI:")) {
            Thread thread = new Thread(new Runnable() {
              public void run() {
                try {
                  Concept local_concept = index_lookup.findConceptByAui(ui.substring(4));
                  clm.addConcept(local_concept);
                  RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
                } catch (Exception e) {
                  e.printStackTrace();
                  MMSToolkit
                      .notifyUser(
                          RRFBrowserGUI.getRRFBrowserGUI(),
                          "Unexpected error looking concept up by this AUI:"
                              + ui
                              + "\n"
                              + "The most likely explanation is that your subset is missing\n"
                              + "an indexes/AUI_MRCONSO.x index file. \n"
                              + "See the console for more information.");
                  return;
                }
              }
            });
            RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
            thread.start();
          } else if (ui.startsWith("Back")) {
            // change the range of records to be displayed
            String key = ui.substring("Back".length());
            int record = ((Integer) expanded_hm.get(key)).intValue();
            if (record > page_size)
              record = record - page_size;
            else
              record = 0;
            expanded_hm.put(key, new Integer(record));

            Thread thread = new Thread(new Runnable() {
              public void run() {
                ExpandableRawView.ui = ui;
                redrawConcept(concept);
                RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
              }
            });
            RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
            thread.start();
          } else if (ui.startsWith("Forward")) {
            // change the range of records to be displayed
            String key = ui.substring("Forward".length());
            int record = ((Integer) expanded_hm.get(key)).intValue();
            record = record + page_size;
            expanded_hm.put(key, new Integer(record));

            Thread thread = new Thread(new Runnable() {
              public void run() {
                ExpandableRawView.ui = ui;
                redrawConcept(concept);
                RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
              }
            });
            RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
            thread.start();

          } else if (expanded_hm.containsKey(ui)) {
            expanded_hm.remove(ui);
            Thread thread = new Thread(new Runnable() {
              public void run() {
                redrawConcept(concept);
                RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
              }
            });
            RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
            thread.start();

          } else if (!expanded_hm.containsKey(ui)) {
            Thread thread = new Thread(new Runnable() {
              public void run() {
                // if (queryToRedraw()) {
                expanded_hm.put(ui, new Integer(0));
                redrawConcept(concept);
                // }
                RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
              }
            });
            RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
            thread.start();

          }
        }
      }
    });

    report_pane.setEditable(false);
    report_pane.setContentType(getContentType());
    JScrollPane sp = new JScrollPane(report_pane,
        ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    report_panel.add(sp, BorderLayout.CENTER);

    // Create the popup menu.
    JPopupMenu popup = new JPopupMenu();
    JMenuItem option_menu_item = new JMenuItem("Options...");
    option_menu_item.addActionListener(this);
    popup.add(option_menu_item);

    popup.addSeparator();
    
    JMenuItem copy_item = new JMenuItem("Copy");
    copy_item.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          Thread thread = new Thread(
            new Runnable() {
              public void run() {
                try {
                  report_pane.copy();
                } catch (Exception e) {
                  e.printStackTrace();
                  MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
                  "Unexpected error expanding report.\n" +
                  "See the console for more information.", false);
                }
                RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
              }
            }
          );
          RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
          thread.start();
        }
      }
    );
    popup.add(copy_item);
    
    popup.addSeparator();

    JMenuItem expand_tree_item = new JMenuItem("Expand Complete Tree");
    expand_tree_item.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Thread thread = new Thread(new Runnable() {
          public void run() {
            try {
              expandAllNodes();
            } catch (Exception e) {
              e.printStackTrace();
              MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
                  "Unexpected error expanding report.\n"
                      + "See the console for more information.", false);
            }
            RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
          }
        });
        RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
        thread.start();
      }
    });

    popup.add(expand_tree_item);
    JMenuItem collapse_tree_item = new JMenuItem("Collapse Complete Tree");
    collapse_tree_item.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Thread thread = new Thread(new Runnable() {
          public void run() {
            try {
              collapseAllNodes();
            } catch (Exception e) {
              e.printStackTrace();
              MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
                  "Unexpected error collapsing report.\n"
                      + "See the console for more information.", false);
            }
            RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
          }
        });
        RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
        thread.start();
      }
    });

    popup.add(collapse_tree_item);

    // Add listener to components that can bring up popup menus.
    PopupListener popup_listener = new PopupListener(popup);
    report_pane.addMouseListener(popup_listener);

    return report_panel;
  }

  public void actionPerformed(ActionEvent e) {
    JMenuItem source = (JMenuItem) (e.getSource());
    String s = source.getText();
    if (s.equals("Options...")) {
      showOptions();
    }
    if (concept != null) {
      RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();

      Thread t = new Thread(new Runnable() {
        public void run() {

          clm.refresh();
          RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
        }
      });
      t.start();

    }
  }

  /**
   * Returns the content type used for the {@link JEditorPane} display.
   * 
   * @return content type
   */
  public String getContentType() {
    return "text/html";
  }

  /**
   * Sets the {@link Concept} to be displayed.
   * 
   * @param concept
   *          {@link Concept}
   */
  public void setConcept(Concept concept) {
    this.concept = concept;
    expanded_hm.clear();
    // default expanded nodes
    expanded_hm.put("MRDEF", null);
    expanded_hm.put("MRCONSO", null);
    expanded_hm.put("MRSTY", null);
    final StringBuffer sb = new StringBuffer(10000);
    
    sb.append("<html><div style=\"font-family:" + font_family + ";font-size:" +
              font_point_size + "pt;  white-space:nowrap\">");
    try {
      rrf = new SingleConceptOutputToStringBuffer();
      final String default_sep = System.getProperty("line.separator");
      System.setProperty("line.separator", "<br>" + default_sep);
      sb.append(getReport(concept));
      System.setProperty("line.separator", default_sep);
      sb.append("</div></html>");
    } catch (Exception e) {
      e.printStackTrace();
      MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
          "Unexpected error while writing concept view."
              + "\n See console for more details.", false);
    }


    try {
      SwingUtilities.invokeAndWait(new Runnable() {
        public void run() {
          report_pane.setText(sb.toString());
          report_pane.requestFocus();
          report_pane.setCaretPosition(0);
        }
      });
    } catch (Exception e) {
      e.printStackTrace();
      MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
          "Unexpected error refreshing report.\n"
              + "See the console for more information.", false);
    }
  }

  /**
   * Indicates the {@link FinderHandler} that will execute the find operation
   * for this {@link ConceptView}
   * 
   * @return {@link FinderHandler}
   */
  public FinderHandler getFinderHandler() {
    JEditorPaneFinderHandler handler = new JEditorPaneFinderHandler(report_pane);
    handler.setDialog(RRFBrowserGUI.getFindDialog());
    return handler;
  }

  /**
   * Sets the {@link Concept} to be displayed.
   * 
   * @param concept
   *          {@link Concept}
   */
  private void redrawConcept(Concept concept) {
    this.concept = concept;
    final StringBuffer sb = new StringBuffer(10000);
    sb.append("<html><div style=\"font-family:" + font_family + ";font-size:" +
              font_point_size + "pt;  white-space:nowrap\">");
    try {
      final String default_sep = System.getProperty("line.separator");
      System.setProperty("line.separator", "<br>" + default_sep);
      sb.append(getReport(concept));
      System.setProperty("line.separator", default_sep);
      sb.append("</div></html>");
      report_pane.setText(sb.toString());
      SwingUtilities.invokeAndWait(new Runnable() {
        public void run() {
          report_pane.scrollRectToVisible(rect);
        }
      });
    } catch (Exception e) {
      e.printStackTrace();
      MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
          "Unexpected error refreshing report.\n"
              + "See the console for more information.", false);
    }
  }

  private void expandAllNodes() {
    expanded_hm.put("ALL", null);
    rect = report_pane.getVisibleRect();

    // if (queryToRedraw())
    redrawConcept(concept);
    // else
    // expanded_hm.remove("ALL");

    try {
      SwingUtilities.invokeAndWait(new Runnable() {
        public void run() {
          report_pane.requestFocus();
          report_pane.setCaretPosition(0);
        }
      });
    } catch (Exception e) {
      e.printStackTrace();
      MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
          "Unexpected error refreshing report.\n"
              + "See the console for more information.", false);
    }
  }

  private void collapseAllNodes() {
    expanded_hm.clear();
    rect = report_pane.getVisibleRect();
    redrawConcept(concept);

    try {
      SwingUtilities.invokeAndWait(new Runnable() {
        public void run() {
          report_pane.requestFocus();
          report_pane.setCaretPosition(0);
        }
      });
    } catch (Exception e) {
      e.printStackTrace();
      MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
          "Unexpected error refreshing report.\n"
              + "See the console for more information.", false);
    }
  }

  /**
   * Resets the configuration back to default values. When used with a GUI this
   * method should change the data model behind the GUI and allow
   * <code>setPanelSelections</code> to actually update the GUI.
   */
  public void resetToDefaults() {
    report_pane.setText("");
    expanded_hm.clear();
  }

  /**
   * Adds a {@link DataChangeListener} that responds to configuration changes.
   * Allows main GUI to record all changes to this gui in the undo/redo
   * mechanism.
   * 
   * @param dcl
   *          the {@link DataChangeListener} to add
   */
  public void addDataChangeListener(DataChangeListener dcl) {

  }

  /**
   * Removes the specified {@link DataChangeListener}.
   * 
   * @param dcl
   *          the {@link DataChangeListener} to remove
   */
  public void removeDataChangeListener(DataChangeListener dcl) {

  }

  /**
   * Method returns all properties specific to the {@link ConceptView}.
   * 
   * @return {@link Properties} only those directly related to the concept view.
   */
  public Properties getProperties() {
    Properties new_filter_props = new Properties();
    new_filter_props
        .setProperty("page_size", new Integer(page_size).toString());

    return new_filter_props;
  }

  /**
   * Sets the view properties.
   * @param p
   *          {@link Properties}
   */
  public void setProperties(Properties p) {
    Properties filter_props = p;
    if (filter_props.containsKey("page_size")) {
      setPageSize(new Integer((String) filter_props.get("page_size"))
          .intValue());
    }
    // redrawConcept(concept);
  }

  /**
   * Returns the name.
   * 
   * @return the name
   */
  public String getName() {
    return "Raw View";
  }

  /**
   * Sets the page size. This is the number of records that will be displayed at
   * one time for nodes with a lot of data.
   * 
   * @param page_size
   *          int
   */
  public void setPageSize(int page_size) {
    this.page_size = page_size;
  }

  /**
   * Presents an advanced options GUI to the user.
   */
  public void showOptions() {
    final ExpandableRawViewOptionsGUI raw_view_gui = 
      new ExpandableRawViewOptionsGUI(this);
    raw_view_gui.setVisible(true);
  }

  /**
   * Indicates whether or not an options menu item should be supplied for
   * advanced configuration options.
   * 
   * @return <code>true</code> if so; <code>false</code> otherwise
   */
  public boolean hasAdvancedOptions() {
    return true;
  }

  /**
   * Prints the concept report.
   * 
   * @throws PrinterException
   */
  public void print() throws PrinterException {
    HTMLDocumentRenderer hdr = new HTMLDocumentRenderer();
    hdr.setTitle("Concept: " + concept.getCui());
    JEditorPane editor = new JEditorPane();
    editor.setContentType("text/html");
    String s = SystemToolkit.removeLinks(report_pane.getText());
    editor.setText(s.replaceFirst("<div>","<div style=\"font-family:" + font_family + ";font-size:" + font_point_size + "pt\">"));
    editor.setDoubleBuffered(false);
    hdr.print(editor);
  }

  /**
   * Returns the {@link Printable} that handles printing.
   * 
   * @return {@link Printable} that handles printing
   */
  public Printable getPrintable() {
    JEditorPane editor = new JEditorPane();
    editor.setContentType("text/html");
    String s = SystemToolkit.removeLinks(report_pane.getText());
    editor.setText(s.replaceFirst("<div>","<div style=\"font-family:" + font_family + ";font-size:" + font_point_size + "pt\">"));
    editor.setDoubleBuffered(false);
    return new OnePagePrintable(editor, "Concept: " + concept.getCui());
  }

  private void addPageIcons(StringBuffer sb, String key, int low_range,
      int high_range, boolean at_high_end) {

    sb.append("<font color=\"green\">");

    if (expanded_hm.containsKey("Full" + key))
      sb.append("[all:").append("<a href=\"" + "Full" + key + "\">page</a>")
          .append("]");
    else {
      sb.append("[").append("<a href=\"" + "Full" + key + "\">all</a>").append(
          ":page]");

      if (low_range == 0)
        sb.append("&nbsp;").append("[prev:");
      else
        sb.append("&nbsp;").append(
            "[" + "<a href=\"" + "Back" + key + "\">prev</a>" + ":");
      if (at_high_end)
        sb.append("next]");
      else
        sb.append("<a href=\"" + "Forward" + key + "\">next</a>" + "]");

      sb.append("&nbsp;").append("(records ").append(low_range + 1).append(
          " to ").append(high_range).append(")");
    }
    sb.append(line_end);
    sb.append("</font>");
  }

  private int getLowestDisplayedRecord(String key) {
    if (expanded_hm.containsKey(key)) {
      Object value = expanded_hm.get(key);
      if (value == null)
        return 0;

      int int_value = ((Integer) value).intValue();
      if (int_value < 0)
        return 0;
      else
        return int_value;
    } else {
      return 0;
    }
  }

  /**
   * This method takes a concept object and produces a concept report from it.
   * This concept report is then returned as a string.
   * 
   * @param concept
   *          An object {@link Concept}.
   * @return An object {@link String} representation of report.
   * @throws Exception
   */
  private String getReport(Concept concept) throws Exception {

    report = new StringBuffer(10000);

    if (getContentType().equals("text/html")) {
      bold_begin = "<b>";
      bold_end = "</b>";
    }

    //
    // MRCONSO Section
    //
    if (concept.getAtoms().length != 0) {
      if (expanded_hm.containsKey("MRCONSO") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRCONSO", null);
        report.append("<a href=\"" + "MRCONSO" + "\"><img src = \"" + minus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRCONSO").append(bold_end).append(" (").
            append(concept.getAtoms().length).append(")").append(line_end);
        rrf.writeMRCONSOSection(concept, report);
        report.append(line_end);
      } else {
        report.append(
            "<a href=\"" + "MRCONSO" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRCONSO").append(bold_end).append(
            " (").append(concept.getAtoms().length).append(")")
            .append(line_end);
      }
    }

    //
    // MRSAT Section
    //
    int total_atts = concept.getAttributes().length;
    if (total_atts > 0) {
      if (expanded_hm.containsKey("MRSAT") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRSAT", null);
        report.append("<a href=\"" + "MRSAT" + "\"><img src = \"" + minus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRSAT").append(bold_end).append(" (").
            append(total_atts).append(")").append(
            line_end);
        int low_range = 0;
        int high_range = total_atts;
        if (total_atts > page_size) {
          if (!expanded_hm.containsKey("FullMRSAT")) {
            low_range = getLowestDisplayedRecord("MRSAT");
            high_range = low_range + page_size;
            if (high_range > total_atts)
              high_range = total_atts;
          }
          addPageIcons(report, "MRSAT", low_range, high_range,
              high_range >= total_atts);
        }
        rrf.writeMRSATSection(concept, report, low_range, high_range);

        if (total_atts > page_size) {
          addPageIcons(report, "MRSAT", low_range, high_range,
              high_range >= total_atts);
        }
        report.append(line_end);

      } else {
        report.append(
            "<a href=\"" + "MRSAT" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRSAT").append(bold_end).append(" (")
            .append(total_atts).append(")").append(line_end);
      }
    }

    //
    // MRDEF Section
    //
    if (concept.getDefinitions().length != 0) {
      if (expanded_hm.containsKey("MRDEF") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRDEF", null);
        report.append("<a href=\"" + "MRDEF" + "\"><img src = \"" + minus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRDEF").append(bold_end).append(
            line_end);
        rrf.writeMRDEFSection(concept, report);
        report.append(line_end);
      }
      else {
        report.append("<a href=\"" + "MRDEF" + "\"><img src = \"" + plus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRDEF").append(bold_end).append(
            line_end);
      }
    }

    //
    // MRSTY Section
    //
    if (concept.getSemanticTypes().length != 0) {
      if (expanded_hm.containsKey("MRSTY") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRSTY", null);
        report.append("<a href=\"" + "MRSTY" + "\"><img src = \"" + minus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRSTY").append(bold_end).append(
            line_end);
        rrf.writeMRSTYSection(concept, report);
        report.append(line_end);
      }
      else {
        report.append("<a href=\"" + "MRSTY" + "\"><img src = \"" + plus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRSTY").append(bold_end).append(
            line_end);
      }
    }

    //
    // MRHIST section
    //
    if (concept.getComponentHistories().length != 0) {
      if (expanded_hm.containsKey("MRHIST") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRHIST", null);
        report.append("<a href=\"" + "MRHIST" + "\"><img src = \"" + minus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRHIST").append(bold_end).append(" (").
            append(concept.getComponentHistories().length).append(")").append(
            line_end);
        rrf.writeMRHISTSection(concept, report);
        report.append(line_end);
      } else {
        report.append(
            "<a href=\"" + "MRHIST" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRHIST").append(bold_end)
            .append(" (").append(concept.getComponentHistories().length)
            .append(")").append(line_end);
      }
    }

    //
    // MRREL Section
    //
    int total_rels = concept.getRelationshipsAsList().size();
    if (concept.getRelationships().length != 0) {
      if (expanded_hm.containsKey("MRREL") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRREL", null);
        report.append("<a href=\"" + "MRREL" + "\"><img src = \"" + minus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRREL").append(bold_end).append(" (").
            append(concept.getRelationships().length).append(")").append(
            line_end);
        int low_range = 0;
        int high_range = total_rels;
        if (total_rels > page_size) {
          if (!expanded_hm.containsKey("FullMRREL")) {
            low_range = getLowestDisplayedRecord("MRREL");
            high_range = low_range + page_size;
            if (high_range > total_rels)
              high_range = total_rels;
          }
          addPageIcons(report, "MRREL", low_range, high_range,
              high_range >= total_rels);
        }
        rrf.writeMRRELSection(concept, report, low_range, high_range);

        if (total_rels > page_size) {
          addPageIcons(report, "MRREL", low_range, high_range,
              high_range >= total_rels);
        }
        report.append(line_end);

      } else {
        report.append(
            "<a href=\"" + "MRREL" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRREL").append(bold_end).append(" (")
            .append(total_rels).append(")").append(line_end);
      }
    }

    //
    // MRCOC section
    //
    int total_cocs = concept.getCoocurrencesAsList().size();
    if (concept.getCoocurrences().length != 0) {
      if (expanded_hm.containsKey("MRCOC") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRCOC", null);
        report.append("<a href=\"" + "MRCOC" + "\"><img src = \"" + minus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRCOC").append(bold_end).append(" (").
            append(concept.getCoocurrences().length).append(")").append(
            line_end);
        int low_range = 0;
        int high_range = total_cocs;
        if (total_cocs > page_size) {
          if (!expanded_hm.containsKey("FullMRCOC")) {
            low_range = getLowestDisplayedRecord("MRCOC");
            high_range = low_range + page_size;
            if (high_range > total_cocs)
              high_range = total_cocs;
          }
          addPageIcons(report, "MRCOC", low_range, high_range,
              high_range >= total_cocs);
        }
        rrf.writeMRCOCSection(concept, report, low_range, high_range);

        if (total_cocs > page_size) {
          addPageIcons(report, "MRCOC", low_range, high_range,
              high_range >= total_cocs);
        }
        report.append(line_end);

      } else {
        report.append(
            "<a href=\"" + "MRCOC" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRCOC").append(bold_end).append(" (")
            .append(total_cocs).append(")").append(line_end);
      }
    }

    //
    // MRCXT section
    //
    int total_cxts = concept.getContextsAsList().size();
    StringBuffer file_buffer = new StringBuffer();
    rrf.writeMRCXTSection(concept, file_buffer, 0, -1);
    if (!file_buffer.toString().equals("")) {
      String[] tokens = FieldedStringTokenizer.split(file_buffer.toString(),
          "\n");

      if (concept.getContexts().length != 0) {
        if (expanded_hm.containsKey("MRCXT") || expanded_hm.containsKey("ALL")) {
          if (expanded_hm.containsKey("ALL"))
            expanded_hm.put("MRCXT", null);
          report.append("<a href=\"" + "MRCXT" + "\"><img src = \"" + minus_url +
                        "\"  border=0></a>").append("&nbsp;");
          report.append(bold_begin).append("MRCXT").append(bold_end).append(
              " (").append(tokens.length).append(")").append(line_end);
          int low_range = 0;
          int high_range = total_cxts;
          if (total_cxts > page_size) {
            if (!expanded_hm.containsKey("FullMRCXT")) {
              low_range = getLowestDisplayedRecord("MRCXT");
              high_range = low_range + page_size;
              if (high_range > total_cxts)
                high_range = total_cxts;
            }
            addPageIcons(report, "MRCXT", low_range, high_range,
                high_range >= total_cxts);
          }
          rrf.writeMRCXTSection(concept, report, low_range, high_range);

          if (total_cxts > page_size) {
            addPageIcons(report, "MRCXT", low_range, high_range,
                high_range >= total_cxts);
          }
          report.append(line_end);

        }
        else {
          report.append("<a href=\"" + "MRCXT" + "\"><img src = \"" + plus_url +
                        "\"  border=0></a>").append("&nbsp;");
          report.append(bold_begin).append("MRCXT").append(bold_end).append(
              " (").append(tokens.length).append(")").append(line_end);
        }
      }
    }

    //
    // MRHIER section
    //
    file_buffer = new StringBuffer();
    rrf.writeMRHIERSection(concept, file_buffer);
    if (!file_buffer.toString().equals("")) {
      if (concept.getContexts().length != 0) {
        if (expanded_hm.containsKey("MRHIER") || expanded_hm.containsKey("ALL")) {
          if (expanded_hm.containsKey("ALL"))
            expanded_hm.put("MRHIER", null);
          report.append("<a href=\"" + "MRHIER" + "\"><img src = \"" + minus_url +
                        "\"  border=0></a>").append("&nbsp;");
          report.append(bold_begin).append("MRHIER").append(bold_end).append(" (").
            append(concept.getContexts().length).append(")").append(
              line_end);
          report.append(file_buffer);
          report.append(line_end);
        }
        else {
          report.append("<a href=\"" + "MRHIER" + "\"><img src = \"" + plus_url +
                        "\"  border=0></a>").append("&nbsp;");
          report.append(bold_begin).append("MRHIER").append(bold_end).append(" (").
            append(concept.getContexts().length).append(")").append(
              line_end);
        }
      }
    }

    //
    // MRMAP section
    //
    file_buffer = new StringBuffer();
    int total_mappings = concept.getMappingsAsList().size();
    if (total_mappings > 0) {
      if (expanded_hm.containsKey("MRMAP") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRMAP", null);
        report.append("<a href=\"" + "MRMAP" + "\"><img src = \"" + minus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRMAP").append(bold_end).append(" (").
            append(total_mappings).append(")").append(
            line_end);

        int low_range = 0;
        int high_range = total_mappings;
        if (total_mappings > page_size) {
          if (!expanded_hm.containsKey("FullMRMAP")) {
            low_range = getLowestDisplayedRecord("MRMAP");
            high_range = low_range + page_size;
            if (high_range > total_mappings)
              high_range = total_mappings;
          }
          addPageIcons(file_buffer, "MRMAP", low_range, high_range,
              high_range >= total_mappings);
        }
        rrf.writeMRMAPSection(concept, file_buffer, low_range, high_range);
        report.append(file_buffer);
        if (total_mappings > page_size) {
          addPageIcons(report, "MRMAP", low_range, high_range,
              high_range >= total_mappings);
        }
        report.append(line_end);
      } else {
        report.append(
            "<a href=\"" + "MRMAP" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRMAP").append(bold_end).append(" (")
            .append(total_mappings).append(")").append(line_end);
      }
    }

    //
    // MRSMAP section
    //
    file_buffer = new StringBuffer();

    if (total_mappings > 0) {
      if (expanded_hm.containsKey("MRSMAP") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRSMAP", null);
        report.append("<a href=\"" + "MRSMAP" + "\"><img src = \"" + minus_url +
                      "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRSMAP").append(bold_end).append(" (").
            append(total_mappings).append(")").append(
            line_end);

        int low_range = 0;
        int high_range = total_mappings;
        if (total_mappings > page_size) {
          if (!expanded_hm.containsKey("FullMRSMAP")) {
            low_range = getLowestDisplayedRecord("MRSMAP");
            high_range = low_range + page_size;
            if (high_range > total_mappings)
              high_range = total_mappings;
          }
          addPageIcons(file_buffer, "MRSMAP", low_range, high_range,
              high_range >= total_mappings);
        }
        rrf.writeMRSMAPSection(concept, file_buffer, low_range, high_range);
        report.append(file_buffer);
        if (total_mappings > page_size) {
          addPageIcons(report, "MRSMAP", low_range, high_range,
              high_range >= total_mappings);
        }

        report.append(line_end);
      } else {
        report.append(
            "<a href=\"" + "MRSMAP" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("MRSMAP").append(bold_end)
            .append(" (").append(total_mappings).append(")").append(line_end);
      }
    }

    report.append(line_end).append("<hr align=\"center\">").append(line_end);

    //
    // MRAUI section
    //
    file_buffer = new StringBuffer();

    if (expanded_hm.containsKey("MRAUI") || expanded_hm.containsKey("ALL")) {
      if (expanded_hm.containsKey("ALL"))
        expanded_hm.put("MRAUI", null);
      report.append("<a href=\"" + "MRAUI" +
                                           "\"><img src = \"" + minus_url +
                                           "\"  border=0></a>").append("&nbsp;");
      report.append(bold_begin).append("MRAUI").append(bold_end).append(" (Find rows matching CUI2)").append(
          line_end);
      if (release_metadata == null) {
        RichMRMetamorphoSysInputStream input_stream = (RichMRMetamorphoSysInputStream) user_configuration
            .getInputStream();

        try {
          input_stream.open(user_configuration.getSourcePaths());
          release_metadata = input_stream.readReleaseMetadata();
          input_stream.close();
        } catch (IOException ioe) {
          MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(), ioe
              + "Error reading release metadata.", false);
        }
      }
      rrf.writeMRAUISection(file_buffer);
      if (!file_buffer.toString().equals(""))
        report.append(file_buffer);
      else
        report.append("No entries.");
      report.append(line_end);
    } else {
      report.append(
          "<a href=\"" + "MRAUI" + "\"><img src = " + plus_url
              + " border=0></a>").append("&nbsp;");
      report.append(bold_begin).append("MRAUI").append(bold_end).append(
          " (Find rows matching CUI2)").append(line_end);
    }
    
    //
    // MRCUI section
    //
    file_buffer = new StringBuffer();
    if (expanded_hm.containsKey("MRCUI") || expanded_hm.containsKey("ALL")) {
      if (expanded_hm.containsKey("ALL"))
        expanded_hm.put("MRCUI", null);
      report.append("<a href=\"" + "MRCUI" +
                                           "\"><img src = \"" + minus_url +
                                           "\"  border=0></a>").append("&nbsp;");
      report.append(bold_begin).append("MRCUI").append(bold_end).append(" (Find rows matching CUI2)").append(
          line_end);
      // Get release metadata
      if (release_metadata == null) {
        RichMRMetamorphoSysInputStream input_stream = (RichMRMetamorphoSysInputStream) user_configuration
            .getInputStream();

        try {
          input_stream.open(user_configuration.getSourcePaths());
          release_metadata = input_stream.readReleaseMetadata();
          input_stream.close();
        } catch (IOException ioe) {
          MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(), ioe
              + "Error reading release metadata.", false);
        }
      }
      rrf.writeMRCUISection(file_buffer);
      if (!file_buffer.toString().equals(""))
        report.append(file_buffer);
      else
        report.append("No entries.");
      report.append(line_end);
    } else {
      report.append(
          "<a href=\"" + "MRCUI" + "\"><img src = " + plus_url
              + " border=0></a>").append("&nbsp;");
      report.append(bold_begin).append("MRCUI").append(bold_end).append(
          " (Find rows matching CUI2)").append(line_end);
    }

    report.append(line_end).append("<hr align=\"center\">").append(line_end);

    //
    // MRCOLS section
    //
    file_buffer = new StringBuffer();
    rrf.writeMRCOLSSection(file_buffer);
    if (!file_buffer.toString().equals("")) {
      String[] tokens = FieldedStringTokenizer.split(file_buffer.toString(),
          "\n");
      if (expanded_hm.containsKey("MRCOLS") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRCOLS", null);
        report.append("<a href=\"" + "MRCOLS" +
                                             "\"><img src = \"" + minus_url +
                                             "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("Full MRCOLS").append(bold_end).append(" (").
            append(tokens.length).append(")").append(
            line_end);
        int low_range = 0;
        int high_range = tokens.length;
        if (tokens.length > page_size) {
          if (!expanded_hm.containsKey("FullMRCOLS")) {
            low_range = getLowestDisplayedRecord("MRCOLS");
            high_range = low_range + page_size;
            if (high_range > tokens.length)
              high_range = tokens.length;
          }
          file_buffer = new StringBuffer();
          addPageIcons(file_buffer, "MRCOLS", low_range, high_range,
              high_range >= tokens.length);

          rrf.writeMRCOLSSection(concept, file_buffer, low_range, high_range);

        }

        report.append(file_buffer);
        if (tokens.length > page_size) {
          addPageIcons(report, "MRCOLS", low_range, high_range,
              high_range >= tokens.length);
        }

        report.append(line_end);
      } else {
        report.append(
            "<a href=\"" + "MRCOLS" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("Full MRCOLS").append(bold_end)
            .append(" (").append(tokens.length).append(")").append(line_end);
      }
    }

    //
    // MRDOC section
    //
    file_buffer = new StringBuffer();
    rrf.writeMRDOCSection(file_buffer);
    if (!file_buffer.toString().equals("")) {
      String[] tokens = FieldedStringTokenizer.split(file_buffer.toString(),
          "\n");
      if (expanded_hm.containsKey("MRDOC") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRDOC", null);
        report.append("<a href=\"" + "MRDOC" +
                                             "\"><img src = \"" + minus_url +
                                             "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("Full MRDOC").append(bold_end).append(" (").
            append(tokens.length).append(")").append(
            line_end);
        int low_range = 0;
        int high_range = tokens.length;
        if (tokens.length > page_size) {
          if (!expanded_hm.containsKey("FullMRDOC")) {
            low_range = getLowestDisplayedRecord("MRDOC");
            high_range = low_range + page_size;
            if (high_range > tokens.length)
              high_range = tokens.length;
          }
          file_buffer = new StringBuffer();
          addPageIcons(file_buffer, "MRDOC", low_range, high_range,
              high_range >= tokens.length);

          rrf.writeMRDOCSection(concept, file_buffer, low_range, high_range);

        }

        report.append(file_buffer);
        if (tokens.length > page_size) {
          addPageIcons(report, "MRDOC", low_range, high_range,
              high_range >= tokens.length);
        }

        report.append(line_end);
      } else {
        report.append(
            "<a href=\"" + "MRDOC" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("Full MRDOC").append(bold_end).append(
            " (").append(tokens.length).append(")").append(line_end);
      }
    }

    //
    // MRFILES section
    //
    file_buffer = new StringBuffer();
    rrf.writeMRFILESSection(file_buffer);
    if (!file_buffer.toString().equals("")) {
      String[] tokens = FieldedStringTokenizer.split(file_buffer.toString(),
          "\n");
      if (expanded_hm.containsKey("MRFILES") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRFILES", null);
        report.append("<a href=\"" + "MRFILES" +
                                             "\"><img src = \"" + minus_url +
                                             "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("Full MRFILES").append(bold_end).append(" (").
            append(tokens.length).append(")").append(
            line_end);
        int low_range = 0;
        int high_range = tokens.length;
        if (tokens.length > page_size) {
          if (!expanded_hm.containsKey("FullMRFILES")) {
            low_range = getLowestDisplayedRecord("MRFILES");
            high_range = low_range + page_size;
            if (high_range > tokens.length)
              high_range = tokens.length;
          }
          file_buffer = new StringBuffer();
          addPageIcons(file_buffer, "MRFILES", low_range, high_range,
              high_range >= tokens.length);

          rrf.writeMRFILESSection(concept, file_buffer, low_range, high_range);

        }

        report.append(file_buffer);
        if (tokens.length > page_size) {
          addPageIcons(report, "MRFILES", low_range, high_range,
              high_range >= tokens.length);
        }

        report.append(line_end);
      } else {
        report.append(
            "<a href=\"" + "MRFILES" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("Full MRFILES").append(bold_end)
            .append(" (").append(tokens.length).append(")").append(line_end);
      }
    }

    //
    // MRRANK section
    //
    file_buffer = new StringBuffer();
    rrf.writeMRRANKSection(file_buffer);
    if (!file_buffer.toString().equals("")) {
      String[] tokens = FieldedStringTokenizer.split(file_buffer.toString(),
          "\n");
      if (expanded_hm.containsKey("MRRANK") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRRANK", null);
        report.append("<a href=\"" + "MRRANK" +
                                             "\"><img src = \"" + minus_url +
                                             "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("Full MRRANK").append(bold_end).append(" (").
            append(tokens.length).append(")").append(
            line_end);
        int low_range = 0;
        int high_range = tokens.length;
        if (tokens.length > page_size) {
          if (!expanded_hm.containsKey("FullMRRANK")) {
            low_range = getLowestDisplayedRecord("MRRANK");
            high_range = low_range + page_size;
            if (high_range > tokens.length)
              high_range = tokens.length;
          }
          file_buffer = new StringBuffer();
          addPageIcons(file_buffer, "MRRANK", low_range, high_range,
              high_range >= tokens.length);

          rrf.writeMRRANKSection(concept, file_buffer, low_range, high_range);

        }

        report.append(file_buffer);
        if (tokens.length > page_size) {
          addPageIcons(report, "MRRANK", low_range, high_range,
              high_range >= tokens.length);
        }

        report.append(line_end);
      } else {
        report.append(
            "<a href=\"" + "MRRANK" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("Full MRRANK").append(bold_end)
            .append(" (").append(tokens.length).append(")").append(line_end);
      }
    }

    //
    // MRSAB section
    //
    file_buffer = new StringBuffer();
    rrf.writeMRSABSection(file_buffer);
    if (!file_buffer.toString().equals("")) {
      String[] tokens = FieldedStringTokenizer.split(file_buffer.toString(),
          "\n");
      if (expanded_hm.containsKey("MRSAB") || expanded_hm.containsKey("ALL")) {
        if (expanded_hm.containsKey("ALL"))
          expanded_hm.put("MRSAB", null);
        report.append("<a href=\"" + "MRSAB" +
                                             "\"><img src = \"" + minus_url +
                                             "\"  border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("Full MRSAB").append(bold_end).append(" (").
            append(tokens.length).append(")").append(
            line_end);
        int low_range = 0;
        int high_range = tokens.length;
        if (tokens.length > page_size) {
          if (!expanded_hm.containsKey("FullMRSAB")) {
            low_range = getLowestDisplayedRecord("MRSAB");
            high_range = low_range + page_size;
            if (high_range > tokens.length)
              high_range = tokens.length;
          }
          file_buffer = new StringBuffer();
          addPageIcons(file_buffer, "MRSAB", low_range, high_range,
              high_range >= tokens.length);

          rrf.writeMRSABSection(concept, file_buffer, low_range, high_range);

        }

        report.append(file_buffer);
        if (tokens.length > page_size) {
          addPageIcons(report, "MRSAB", low_range, high_range,
              high_range >= tokens.length);
        }

        report.append(line_end);
      } else {
        report.append(
            "<a href=\"" + "MRSAB" + "\"><img src = " + plus_url
                + " border=0></a>").append("&nbsp;");
        report.append(bold_begin).append("Full MRSAB").append(bold_end).append(
            " (").append(tokens.length).append(")").append(line_end);
      }
    }

    // Return the report
    return report.toString();
  }

  /**
   * Returns the preferred atom's string from amongst the given {@link Atom}[].
   * Uses the Atom.AtomRankComparator and the given precidence list to do
   * ranking.
   * 
   * @param atoms
   *          {@link Atom}[]
   * @param precedence
   *          {@link List}
   * @return preferred atom's STR string
   */
  public String getPreferredAtom(Atom[] atoms, List precedence) {
    try {
      Arrays.sort(atoms, new Atom.AtomRankComparator(atoms,
          (SourceTermType[]) precedence.toArray(new SourceTermType[0]), true,
          true, true));
    } catch (Exception e) {
      MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(), e.getMessage(),
          false);
      return report.toString();
    }
    return atoms[0].getStr();
  }

  /**
   * Handles opening of a source directory.
   * 
   * @param source_dir
   *          the source directory
   */
  public void subsetDirectoryOpened(File source_dir) {
    try {
      index_lookup = new RichMRFilesIndexLookup(user_configuration
          .getSourcePaths()[0]);
    } catch (IOException ioe) {
      ioe.printStackTrace();
      MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
          "Unexpected error opening the subset directory.\n"
              + "See the console for more information.", false);
    }
    release_metadata = null;
  }

  public class PopupListener extends MouseAdapter {
    JPopupMenu popup;

    PopupListener(JPopupMenu popupMenu) {
      popup = popupMenu;
    }

    public void mousePressed(MouseEvent e) {
      maybeShowPopup(e);
    }

    public void mouseReleased(MouseEvent e) {
      maybeShowPopup(e);
    }

    private void maybeShowPopup(MouseEvent e) {
      if (e.isPopupTrigger()) {
        popup.show(e.getComponent(), e.getX(), e.getY());
      }
    }
  }

  //
  // Inner Class
  //

  /**
   * Extends {@link RichMRFilesConceptOutputStream} and allows a single concept
   * to be printed out to a {@link StringBuffer}.
   */
  private class SingleConceptOutputToStringBuffer extends
      RichMRFilesConceptOutputStream {

    /**
     * Writes the given {@link Concept} to the given {@link StringBuffer}.
     * 
     * @param c
     *          {@link Concept}
     * @param sb
     *          {@link StringBuffer}
     * @throws IOException
     */
    private void writeMRCONSOSection(Concept c, StringBuffer sb)
        throws IOException {
      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRCONSO", new BufferedWriter(string_writer));
      writeMRCONSO(c);
      flush("MRCONSO");
      String s = string_writer.toString();
      sb.append(s);
    }

    private void writeMRSATSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {
      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRSAT", new BufferedWriter(string_writer));
      Concept full_concept = c;
      Concept dummy_concept = new ConceptImpl(full_concept);
      Attribute[] all_atts = full_concept.getAttributes();
      Attribute[] subset_atts = new Attribute[high_range - low_range];
      int index = 0;
      for (int i = low_range; i < high_range; i++) {
        subset_atts[index++] = all_atts[i];
      }
      dummy_concept.clearAttributes();
      dummy_concept.setAttributes(subset_atts);

      writeMRSAT(dummy_concept);
      flush("MRSAT");
      sb.append(string_writer.toString());
    }

    private void writeMRDEFSection(Concept c, StringBuffer sb)
        throws IOException {
      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRDEF", new BufferedWriter(string_writer));
      writeMRDEF(c);
      flush("MRDEF");
      sb.append(string_writer.toString());
    }

    private void writeMRSTYSection(Concept c, StringBuffer sb)
        throws IOException {
      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRSTY", new BufferedWriter(string_writer));
      writeMRSTY(c);
      flush("MRSTY");
      sb.append(string_writer.toString());
    }

    private void writeMRHISTSection(Concept c, StringBuffer sb)
        throws IOException {
      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRHIST", new BufferedWriter(string_writer));
      writeMRHIST(c);
      flush("MRHIST");
      sb.append(string_writer.toString());
    }

    private void writeMRRELSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {
      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRREL", new BufferedWriter(string_writer));
      final Relationship[] rels = c.getRelationships();
      final Map rels_a_hm = new HashMap();
      final Map rels_c_hm = new HashMap();
      for (int i = low_range; i < high_range; i++) {
        final Relationship rel = rels[i];
        final String cui1 = concept.getCui();
        final String cui2 = rel.getRelatedConcept().getCui();
        if (rel instanceof AtomRelationship && !cui1.equals(cui2)) {
          final AtomRelationship atom_rel = (AtomRelationship) rel;
          String aui = atom_rel.getRelatedAtom().getAui();
          atom_rel.getRelatedAtom().setAui(
              "<a href=\"AUI:" + aui + "\">" + aui + "</a>");
          rels_a_hm.put(atom_rel, aui);
        }

        if (!cui1.equals(cui2)) {
          rel.getRelatedConcept().setCui(
              "<a href=\"CUI:" + cui2 + "\">" + cui2 + "</a>");
          rels_c_hm.put(rel, cui2);
        }
      }

      Concept full_concept = c;
      Concept dummy_concept = new ConceptImpl(full_concept);
      Relationship[] all_rels = full_concept.getRelationships();
      Relationship[] subset_rels = new Relationship[high_range - low_range];
      int index = 0;
      for (int i = low_range; i < high_range; i++) {
        subset_rels[index++] = all_rels[i];
      }
      dummy_concept.clearRelationships();
      dummy_concept.setRelationships(subset_rels);

      writeMRREL(dummy_concept);
      flush("MRREL");
      sb.append(string_writer.toString());
      // need to set auis and cuis back to normal - no more links
      for (int i = low_range; i < high_range; i++) {
        final Relationship rel = rels[i];
        final String cui1 = concept.getCui();
        final String cui2 = rel.getRelatedConcept().getCui();
        if (!cui1.equals(cui2)) {
          String cui = (String) rels_c_hm.get(rel);
          rel.getRelatedConcept().setCui(cui);
        }
        if (rel instanceof AtomRelationship && !cui1.equals(cui2)) {
          final AtomRelationship atom_rel = (AtomRelationship) rel;
          String aui = (String) rels_a_hm.get(atom_rel);
          atom_rel.getRelatedAtom().setAui(aui);
        }
      }
    }

    private void writeMRCOCSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {
      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRCOC", new BufferedWriter(string_writer));
      final Coocurrence[] cocs = c.getCoocurrences();
      final Map cocs_hm = new HashMap();
      for (int i = low_range; i < high_range; i++) {
        final Coocurrence coc = cocs[i];
        final String cui = coc.getRelatedConcept().getCui();
        if (!cui.equals("") && (cui.indexOf("href") == -1)) {
          coc.getRelatedConcept().setCui("<a href=\"CUI:" + cui + "\">" + cui + "</a>");
          cocs_hm.put(coc.getRelatedConcept(), cui);
          String coa = coc.getCoa();
          String new_coa = coa.replaceAll("<>", "&lt;&gt;");
          coc.setCoa(new_coa);
        }
        if (coc instanceof AtomCoocurrence) {
          final AtomCoocurrence atom_coc = (AtomCoocurrence) coc;
          String aui = atom_coc.getRelatedAtom().getAui();
          if (!aui.equals("") && (aui.indexOf("href") == -1)) {
            atom_coc.getRelatedAtom().setAui(
                "<a href=\"AUI:" + aui + "\">" + aui + "</a>");
            cocs_hm.put(atom_coc.getRelatedAtom(), aui);
          }
        }
      }
      Concept full_concept = c;
      Concept dummy_concept = new ConceptImpl(full_concept);
      Coocurrence[] all_cocs = full_concept.getCoocurrences();
      Coocurrence[] subset_cocs = new Coocurrence[high_range - low_range];
      int index = 0;
      for (int i = low_range; i < high_range; i++) {
        subset_cocs[index++] = all_cocs[i];
      }
      dummy_concept.clearCoocurrences();
      dummy_concept.setCoocurrences(subset_cocs);

      writeMRCOC(dummy_concept);
      flush("MRCOC");
      sb.append(string_writer.toString());
      // need to set auis and cuis back to normal - no more links
      for (int i = low_range; i < high_range; i++) {
        final Coocurrence coc = cocs[i];
        if (cocs_hm.containsKey(coc.getRelatedConcept())) {
          String cui = (String) cocs_hm.get(coc.getRelatedConcept());
          coc.getRelatedConcept().setCui(cui);
        }
        if (cocs[i] instanceof AtomCoocurrence) {
          final AtomCoocurrence atom_coc = (AtomCoocurrence) cocs[i];
          if (atom_coc.getRelatedAtom().getAui().indexOf("href") != -1) {
            String aui = atom_coc.getRelatedAtom().getAui();
            //aui = aui.substring(aui.indexOf("=") + 1, aui.indexOf(">"));
            aui = aui.substring(aui.indexOf(">") + 1, aui.indexOf("<",2));
            atom_coc.getRelatedAtom().setAui(aui);
          }
        }
      }
    }

    private void writeMRCXTSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {

      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRCXT", new BufferedWriter(string_writer));
      final Context[] cxts = c.getContexts();
      if (high_range == -1)
        high_range = cxts.length;
      for (int i = low_range; i < high_range; i++) {
        final Context cxt = cxts[i];
        final ContextMember[] cxt_mems = cxt.getContextMembers();
        for (int j = 0; j < cxt_mems.length; j++) {
          final ContextMember cxt_mem = cxt_mems[j];
          final String cui = cxt_mem.getRelatedConcept().getCui();
          if (!cui.equals("") && (cui.indexOf("href") == -1)) {
            cxt_mem.getRelatedConcept().setCui(
                "<a href=\"CUI:" + cui + ">" + cui + "</a>");
          }
          String aui = cxt_mem.getRelatedAtom().getAui();
          if (!aui.equals("") && (aui.indexOf("href") == -1)) {
            cxt_mem.getRelatedAtom().setAui(
                "<a href=\"AUI:" + aui + ">" + aui + "</a>");
          }
        }
      }
      writeMRCXT(c);
      flush("MRCXT");
      sb.append(string_writer.toString());
      // need to set auis and cuis back to normal - no more links
      for (int i = low_range; i < high_range; i++) {
        final Context cxt = cxts[i];
        final ContextMember[] cxt_mems = cxt.getContextMembers();
        for (int j = 0; j < cxt_mems.length; j++) {
          final ContextMember cxt_mem = cxt_mems[j];
          if (cxt_mem.getRelatedConcept().getCui().indexOf("href") != -1) {
            String cui = cxt_mem.getRelatedConcept().getCui();
            //cui = cui.substring(cui.indexOf("=") + 1, cui.indexOf(">"));
            cui = cui.substring(cui.indexOf(">") + 1, cui.indexOf("<",2));
            cxt_mem.getRelatedConcept().setCui(cui);
          }
          if (cxt_mem.getRelatedAtom().getAui().indexOf("href") != -1) {
            String aui = cxt_mem.getRelatedAtom().getAui();
            //aui = aui.substring(aui.indexOf("=") + 1, aui.indexOf(">"));
            aui = aui.substring(aui.indexOf(">") + 1, aui.indexOf("<",2));
            cxt_mem.getRelatedAtom().setAui(aui);
          }
        }
      }
    }

    private void writeMRHIERSection(Concept c, StringBuffer sb)
        throws IOException {
      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRHIER", new BufferedWriter(string_writer));
      writeMRHIER(c);
      flush("MRHIER");
      sb.append(string_writer.toString());
    }

    private void writeMRMAPSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {
      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRMAP", new BufferedWriter(string_writer));
      Concept full_concept = c;
      Concept dummy_concept = new ConceptImpl(full_concept);
      Mapping[] all_mappings = full_concept.getMappings();
      Mapping[] subset_mappings = new Mapping[high_range - low_range];
      int index = 0;
      for (int i = low_range; i < high_range; i++) {
        subset_mappings[index++] = all_mappings[i];
      }
      dummy_concept.clearMappings();
      dummy_concept.setMappings(subset_mappings);
      writeMRMAP(dummy_concept);
      flush("MRMAP");
      sb.append(string_writer.toString());
    }

    private void writeMRSMAPSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {
      StringWriter string_writer = new StringWriter();
      mr_targets.put("MRSMAP", new BufferedWriter(string_writer));
      Concept full_concept = c;
      Concept dummy_concept = new ConceptImpl(full_concept);
      Mapping[] all_mappings = full_concept.getMappings();
      Mapping[] subset_mappings = new Mapping[high_range - low_range];
      int index = 0;
      for (int i = low_range; i < high_range; i++) {
        subset_mappings[index++] = all_mappings[i];
      }
      dummy_concept.clearMappings();
      dummy_concept.setMappings(subset_mappings);
      writeMRSMAP(dummy_concept);
      flush("MRSMAP");
      sb.append(string_writer.toString());
    }

    //
    // METADATA
    //
    private void writeMRAUISection(StringBuffer sb) throws IOException {
      StringWriter string_writer = new StringWriter();
      BufferedWriter bw = new BufferedWriter(string_writer);
      mr_targets.put("MRAUI", bw);
      writeMRAUI(release_metadata);
      bw.flush();
      sb.append(string_writer.toString());
    }

    private void writeMRCOLSSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {
      String[][] fields = SystemToolkit.readFieldedFile(new File(
          user_configuration.getSourcePaths()[0], "MRCOLS.RRF"), "|");
      fields = truncateFieldedFile(fields, low_range, high_range);
      appendFieldedFile(sb, fields);
    }

    private void writeMRCOLSSection(StringBuffer sb) throws IOException {
      String[][] fields = SystemToolkit.readFieldedFile(new File(
          user_configuration.getSourcePaths()[0], "MRCOLS.RRF"), "|");
      appendFieldedFile(sb, fields);
    }

    private void writeMRCUISection(StringBuffer sb) throws IOException {
      StringWriter string_writer = new StringWriter();
      BufferedWriter bw = new BufferedWriter(string_writer);
      mr_targets.put("MRCUI", bw);
      writeMRCUI(release_metadata);
      bw.flush();
      sb.append(string_writer.toString());
    }

    private void writeMRDOCSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {
      String[][] fields = SystemToolkit.readFieldedFile(new File(
          user_configuration.getSourcePaths()[0], "MRDOC.RRF"), "|");
      fields = truncateFieldedFile(fields, low_range, high_range);
      appendFieldedFile(sb, fields);
    }

    private void writeMRDOCSection(StringBuffer sb) throws IOException {
      String[][] fields = SystemToolkit.readFieldedFile(new File(
          user_configuration.getSourcePaths()[0], "MRDOC.RRF"), "|");
      appendFieldedFile(sb, fields);
    }

    private void writeMRFILESSection(StringBuffer sb) throws IOException {
      String[][] fields = SystemToolkit.readFieldedFile(new File(
          user_configuration.getSourcePaths()[0], "MRFILES.RRF"), "|");
      appendFieldedFile(sb, fields);
    }

    private void writeMRFILESSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {
      String[][] fields = SystemToolkit.readFieldedFile(new File(
          user_configuration.getSourcePaths()[0], "MRFILES.RRF"), "|");
      fields = truncateFieldedFile(fields, low_range, high_range);
      appendFieldedFile(sb, fields);
    }

    private void writeMRRANKSection(StringBuffer sb) throws IOException {
      String[][] fields = SystemToolkit.readFieldedFile(new File(
          user_configuration.getSourcePaths()[0], "MRRANK.RRF"), "|");
      appendFieldedFile(sb, fields);

    }

    private void writeMRRANKSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {
      String[][] fields = SystemToolkit.readFieldedFile(new File(
          user_configuration.getSourcePaths()[0], "MRRANK.RRF"), "|");
      fields = truncateFieldedFile(fields, low_range, high_range);
      appendFieldedFile(sb, fields);
    }

    private void writeMRSABSection(StringBuffer sb) throws IOException {
      String[][] fields = SystemToolkit.readFieldedFile(new File(
          user_configuration.getSourcePaths()[0], "MRSAB.RRF"), "|");
      appendFieldedFile(sb, fields);
    }

    private void writeMRSABSection(Concept c, StringBuffer sb, int low_range,
        int high_range) throws IOException {
      String[][] fields = SystemToolkit.readFieldedFile(new File(
          user_configuration.getSourcePaths()[0], "MRSAB.RRF"), "|");
      fields = truncateFieldedFile(fields, low_range, high_range);
      appendFieldedFile(sb, fields);
    }

    private void appendFieldedFile(StringBuffer sb, String[][] fields) {
      appendFieldedFile(sb, fields, 0, fields.length);
    }

    private void appendFieldedFile(StringBuffer sb, String[][] fields,
        int min_row, int max_row) {
      for (int i = min_row; i < max_row; i++) {
        for (int j = 0; j < fields[i].length; j++) {
          sb.append(fields[i][j]).append("|");
        }
        sb.append(System.getProperty("line.separator"));
      }
    }

    private String[][] truncateFieldedFile(String[][] fields, int min_row,
        int max_row) {
      int x = 0, y = 0;
      String[][] fields_subset = new String[max_row - min_row][fields[0].length];
      for (int i = min_row; i < max_row; i++) {
        for (int j = 0; j < fields[i].length; j++) {
          fields_subset[y][x++] = fields[i][j];
        }
        y++;
        x = 0;
      }
      return fields_subset;
    }

    /**
     * Writes a line to the <code>MRAUI.RRF</code> file.
     * 
     * @param release_metadata
     *          the {@link ReleaseMetadata}
     * @throws IOException
     *           if anything goes wrong
     */
    protected void writeMRAUI(ReleaseMetadata release_metadata)
        throws IOException {

      //
      // Iterate over atom mappings
      //
      List am_list = release_metadata.getAtomMappingsAsList();

      for (int i = 0; i < am_list.size(); i++) {
        //
        // Prepare a line of MRAUI
        //
        final AtomMapping mapping = (AtomMapping) am_list.get(i);
        if (!mapping.getCui2().equals(concept.getCui()))
          continue;
        final StringBuffer sb = new StringBuffer(50);
        sb.append("&nbsp;&nbsp;");
        sb.append(mapping.getAui1());
        sb.append("|");
        sb.append(mapping.getCui1());
        sb.append("|");
        sb.append(mapping.getVer());
        sb.append("|");
        sb.append((mapping.getRel() != null) ? mapping.getRel()
            .getAbbreviation() : "");
        sb.append("|");
        sb.append((mapping.getRela() != null) ? mapping.getRela()
            .getAbbreviation() : "");
        sb.append("|");
        sb.append((mapping.getMapReason() != null) ? mapping.getMapReason()
            : "");
        sb.append("|");
        if (mapping.getAui2() == null || mapping.getAui2().equals("")) {
          sb.append("|||");
        } else {
          sb.append(mapping.getAui2());
          sb.append("|");
          sb.append(mapping.getCui2());
          sb.append("|");
          if (mapping.inCurrentSubset())
            sb.append("Y|");
          else
            sb.append("N|");
        }

        //
        // Write a line and process file statistics
        //
        printLine(sb.toString(), "MRAUI");
      }

      //
      // Sorting of the file now takes place at the end
      //
    }

    /**
     * Writes a line to the <code>MRCUI.RRF</code> file.
     * 
     * @param release_metadata
     *          the {@link ReleaseMetadata}
     * @throws IOException
     *           if anything goes wrong
     */
    protected void writeMRCUI(ReleaseMetadata release_metadata)
        throws IOException {

      //
      // Iterate over concept mappings
      //
      List cm_list = release_metadata.getConceptMappingsAsList();

      for (int i = 0; i < cm_list.size(); i++) {

        //
        // Prepare a line of MRCUI
        //
        final ConceptMapping mapping = (ConceptMapping) cm_list.get(i);
        // only return lines relevant to this concept
        if (!mapping.getCui2().equals(concept.getCui()))
          continue;
        final StringBuffer sb = new StringBuffer(50);
        sb.append("&nbsp;&nbsp;");
        sb.append(mapping.getCui1());
        sb.append("|");
        sb.append(mapping.getVer());
        sb.append("|");
        sb.append((mapping.getRel() != null) ? mapping.getRel()
            .getAbbreviation() : "");
        sb.append("|");
        sb.append((mapping.getRela() != null) ? mapping.getRela()
            .getAbbreviation() : "");
        sb.append("|");
        sb.append((mapping.getReason() != null) ? mapping.getReason() : "");
        sb.append("|");
        if (mapping.getCui2() == null || mapping.getCui2().equals("")) {
          sb.append("||");
        } else {
          sb.append(mapping.getCui2());
          sb.append("|");
          if (mapping.inCurrentSubset())
            sb.append("Y|");
          else
            sb.append("N|");
        }

        //
        // Write a line and process file statistics
        //
        printLine(sb.toString(), "MRCUI");
      }

      //
      // Sorting of the file now takes place at the end
      //
    }
  
    /**
     * Writes MRCONSO lines. Replaces < and >.
     * 
     * @param concept
     *          {@link Concept} to be written
     * @throws IOException
     */
    protected void writeMRCONSO(Concept concept) throws IOException {

      //
      // Iterate through atoms
      //
      Atom[] atoms = concept.getAtoms();
      List mrconso_lines = new ArrayList(atoms.length);
      for (int i = 0; i < atoms.length; i++) {
        //
        // Prepare MRCONSO.RRF line
        //
        final Atom atom = atoms[i];
        final StringBuffer sb = new StringBuffer(150);
        sb.append(atom.getConcept().getCui()).append("|");
        sb.append(atom.getLat().getAbbreviation()).append("|");
        sb.append(atom.getTs().getAbbreviation()).append("|");
        sb.append(atom.getLui()).append("|");
        sb.append(atom.getStt().getAbbreviation()).append("|");
        sb.append(atom.getSui()).append("|");
        sb.append((atom.hasPreferredAtomIdentifier()) ? "Y" : "N").append("|");
        sb.append((atom.getAui() != null) ? atom.getAui() : "").append("|");
        sb.append((atom.getSaui() != null) ? atom.getSaui() : "").append("|");
        sb.append((atom.getScui() != null) ? atom.getScui() : "").append("|");
        sb.append((atom.getSdui() != null) ? atom.getSdui() : "").append("|");
        if (versioned_output)
          sb.append(atom.getSource().getVersionedSourceAbbreviation());
        else
          sb.append(atom.getSource().getRootSourceAbbreviation());
        sb.append("|");
        sb.append(atom.getTty().getAbbreviation()).append("|");
        sb.append(atom.getCode()).append("|");
        sb
            .append(
                atom.getStr().replaceAll("<", "&lt;").replaceAll(">", "&gt;"))
            .append("|");
        sb.append(atom.getSource().getSrl()).append("|");
        sb.append(atom.getSuppress()).append("|");
        sb.append((atom.getCvf() != null) ? atom.getCvf().toString() : "")
            .append("|");

        //
        // Add line to list
        //
        mrconso_lines.add(sb.toString());
      }
      //
      // Sort (unique) lines and write to MRCONSO.RRF
      //
      Collections.sort(mrconso_lines);
      String prev_line = "";
      for (int i = 0; i < mrconso_lines.size(); i++) {
        final String this_line = mrconso_lines.get(i).toString();
        if (!this_line.equals(prev_line))
          printLine(this_line, "MRCONSO");
        prev_line = this_line;
      }
    }

  }

  /**
   * Provides the Report View Options GUI.
   */
  private class ExpandableRawViewOptionsGUI extends JDialog {

    
    //
    // Private Fields
    //
    private JComboBox page_size_display;

    /**
     * Instantiates an {@link ExpandableRawViewOptionsGUI} for the specified
     * {@link ExpandableRawView}.
     * 
     * @param f
     *          the {@link ExpandableRawView}
     */
    public ExpandableRawViewOptionsGUI(ExpandableRawView f) {
      super(RRFBrowserGUI.getRRFBrowserGUI());
      setModal(true);
      setTitle("Raw View Options");
      initialize();
    };

    //
    // Methods
    //

    public void actionPerformed(java.awt.event.ActionEvent e) {

    }

    /**
     * This method sets up the GUI
     */
    private void initialize() {

      final DataChangeListener dcl = new DataChangeListener();

      getContentPane().setLayout(new BorderLayout());

      // configure source_lang_panel
      JPanel source_lang_panel = new JPanel();
      BoxLayout source_lang_layout = new BoxLayout(source_lang_panel,
          BoxLayout.Y_AXIS);
      source_lang_panel.setLayout(source_lang_layout);

      // Add the page size display
      page_size_display = new JComboBox();
      page_size_display.setEditable(false);
      String[] page_size_choices = { "50", "100", "200", "500", "1000" };
      for (int i = 0; i < page_size_choices.length; i++) {
        page_size_display.addItem(page_size_choices[i]);
      }
      if (page_size == 0) {
        page_size_display.setSelectedItem(page_size_choices[1]);
        setPageSize(new Integer(page_size_choices[1]).intValue());
      } else
        page_size_display.setSelectedItem(new String(new Integer(page_size)
            .toString()));

      source_lang_panel.add(new JLabel("\n"));
      JLabel page_label = new JLabel("Select page size for display.");
      source_lang_panel.add(page_label);
      source_lang_panel.add(page_size_display);

      // configure done_panel
      FlowLayout done_layout = new FlowLayout();
      done_layout.setVgap(10);
      JPanel done_panel = new JPanel(done_layout);
      JPanel complete_panel = new JPanel(new BorderLayout(10, 10));

      JButton jdone = new JButton("Done");
      jdone.addActionListener(dcl);
      done_panel.add(jdone);

      // Create an action object
      Action cancelAction = new AbstractAction("Cancel") {
        {
          // Set action command to "Cancel"
          putValue(Action.ACTION_COMMAND_KEY, getValue(Action.NAME));
        }

        // This method is called when the action is invoked
        public void actionPerformed(ActionEvent e) {
          dcl.actionPerformed(e);
        }
      };
      JButton jcancel = new JButton(cancelAction);
      // Register keystroke
      done_panel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
          KeyStroke.getKeyStroke("ESCAPE"), cancelAction.getValue(Action.NAME));
      // Register action
      done_panel.getActionMap().put(cancelAction.getValue(Action.NAME),
          cancelAction);
      done_panel.add(jcancel);

      // Add the sub-panels to complete_panel
      complete_panel.add(source_lang_panel, BorderLayout.CENTER);
      complete_panel.add(done_panel, BorderLayout.SOUTH);
      EtchedBorder etched_border = new EtchedBorder(EtchedBorder.RAISED);
      complete_panel.setBorder(new CompoundBorder(etched_border,
          new EmptyBorder(10, 10, 10, 10)));

      getContentPane().add(complete_panel, BorderLayout.CENTER);
      getRootPane().setDefaultButton(jdone);
      this.pack();
      this.setLocationRelativeTo(null);
    };

    //
    // Inner classes
    //
    public class DataChangeListener implements java.awt.event.ActionListener {

      // constructor
      public DataChangeListener() {
      };

      // implementation of ActionListener interface
      public void actionPerformed(ActionEvent e) {

        if (e.getActionCommand().equals("Done")) {

          ExpandableRawView.this.setPageSize(new Integer(
              (String) page_size_display.getSelectedItem()).intValue());

          if (concept != null) {
            RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();

            Thread t = new Thread(new Runnable() {
              public void run() {

                clm.refresh();
                RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
              }
            });
            t.start();

          }
          ExpandableRawViewOptionsGUI.this.dispose();
        } else if (e.getActionCommand().equals("Cancel"))
          dispose();
      }
    }
  } // End of ExpandableRawViewOptionsGUI class

}
