/*****************************************************************************
 *
 * Object:  ForwardAction.java
 * Author:  Deborah Shapiro
 *
 *****************************************************************************/
package gov.nih.nlm.mms.browser;

import gov.nih.nlm.swing.SwingToolkit;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

/**
 * Action used for forward navigation.
 * 
 * @author Deborah Shapiro (dshapiro@apelon.com)
 */
public class ForwardAction extends AbstractAction {

  //
  // Concept list manager
  //
  private ConceptListManager clm;

  /**
   * Constructor.
   * 
   * @param clm
   *          {@link ConceptListManager}
   */
  public ForwardAction(ConceptListManager clm) {
    super();

    this.clm = clm;

    putValue(Action.NAME, "Forward");
    putValue(Action.MNEMONIC_KEY, new Integer((int) 'f'));
    putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke('F',
    		Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
    putValue(Action.SHORT_DESCRIPTION, "Navigate forward");
    ImageIcon clm_icon = null;
    BufferedImage clm_image = null;
    try {
      URL gif_url = this.getClass().getClassLoader().getResource(
          "config/arrows_blue_005.gif");
      clm_image = SwingToolkit.readScaledImage(gif_url, 17, false);
      clm_icon = new ImageIcon(clm_image);
    } catch (Exception ex) {
      throw new RuntimeException(
          "Scaled version of config/arrows_blue_005.gif could not be loaded.");
    }
    putValue(Action.SMALL_ICON, clm_icon);
    //setEnabled(false);
  }

  /**
   * Action performed.
   * 
   * @param e
   *          {@link ActionEvent}
   */
  public void actionPerformed(ActionEvent e) {
    RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
    Thread thread = new Thread(new Runnable() {
      public void run() {
        clm.forward();
        RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
      }
    });
    thread.start();
  }
}
