/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms.browser;
 * Class:      TreeBrowserFinder.java
 *
 *************************************************************************/
package gov.nih.nlm.mms.browser;

import gov.nih.nlm.mms.GUIConfigurable;
import gov.nih.nlm.mms.MMSToolkit;
import gov.nih.nlm.umls.io.RichMRFilesIndexLookup;
import gov.nih.nlm.umls.meta.Atom;
import gov.nih.nlm.umls.meta.Concept;
import gov.nih.nlm.umls.meta.Context;
import gov.nih.nlm.umls.meta.Source;
import gov.nih.nlm.umls.meta.impl.AtomImpl;
import gov.nih.nlm.umls.meta.impl.ConceptImpl;
import gov.nih.nlm.umls.meta.impl.ContextImpl;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.tree.TreeNode;

/**
 * Represents a {@link Finder} that enables user to choose concepts for display
 * from an HTML document formatted to look like a tree.
 *
 * @author Deborah Shapiro
 *
 */
public class TreeBrowserFinder extends AbstractFinder {
  //
  // GUI
  //
  private JEditorPane report_pane;
  
  //
  // Indicates which nodes have already been expanded
  //
  private Map expanded_hm = null;

  //
  // plus and minus icons for table
  //
  private URL plus_url = null, minus_url = null, blank_url = null; 

  /**
   * Used to perform searches.
   */
  private RichMRFilesIndexLookup index_lookup = null;

  /**
   * Root of tree.
   */
  private ContextNode header = null;

  /**
   * Max number of children.
   */
  private int max_children = 200;
  
  final private String line_end = "<br>"; 
  
  private String indent = "";
  
  private Rectangle rect;
  
  private boolean expanding = false;

  
  /**
   * Instantiates a {@link TreeBrowserFinder}.
   */
  public TreeBrowserFinder() {
    expanded_hm = new HashMap();
    try {
      plus_url = this.getClass().getClassLoader().getResource("config/plus2.gif");
      minus_url = this.getClass().getClassLoader().getResource("config/minus2.gif");
      blank_url = this.getClass().getClassLoader().getResource("config/blank.gif");
    }
    catch (Exception ex) {
      throw new RuntimeException("config/*.gif file could not be loaded.");
    }
  }

  /**
   * Returns a {@link JPanel} containing a {@link JTree} that
   * enables navigation through the context hierarchy.
   * @return a {@link JPanel} containing a {@link JTree} that
   * enables navigation through the context hierarchy
   */
  public JPanel getPanel() {

    //JPanel report_panel = new JPanel();
    //report_panel.setLayout(new BoxLayout(report_panel, BoxLayout.Y_AXIS));
  	final JPanel full_panel = new JPanel(new BorderLayout(5, 5));
    full_panel.setBorder(new EmptyBorder(5, 5, 5, 5));
  	report_pane = new JEditorPane();
    report_pane.addHyperlinkListener(new HyperlinkListener() {
			public void hyperlinkUpdate(HyperlinkEvent he) {
				if (he.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
					final String ui = he.getDescription();
					if (ui.startsWith("CUI:")) {
						Thread thread = new Thread(new Runnable() {
							public void run() {
								Concept local_concept = new ConceptImpl();
								local_concept.setCui(ui.substring(4));
								clm.addConcept(local_concept);
								RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
							}
						});
						RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
						thread.start();
					} else if (expanded_hm.containsKey(ui)) {
						expanded_hm.remove(ui);
						Thread thread = new Thread(new Runnable() {
							public void run() {
								rect = report_pane.getVisibleRect();
								refresh();
								RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
							}
						});
						RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
						thread.start();

					} else if (!expanded_hm.containsKey(ui)) {
						expanded_hm.put(ui, new Integer(0));
						expanding = true;
						Thread thread = new Thread(new Runnable() {
							public void run() {
								rect = report_pane.getVisibleRect();
								refresh();
								RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
							}
						});
						RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
						thread.start();
					}
				}
			}
		});

    report_pane.setEditable(false);
    report_pane.setContentType(getContentType());
    JScrollPane sp = new JScrollPane(report_pane,
                                     ScrollPaneConstants.
                                     VERTICAL_SCROLLBAR_AS_NEEDED,
                                     ScrollPaneConstants.
                                     HORIZONTAL_SCROLLBAR_AS_NEEDED);
    full_panel.add(sp, BorderLayout.CENTER);

    //
    // Create the popup menu.
    //
    JPopupMenu popup = new JPopupMenu();
    JMenuItem option_menu_item = new JMenuItem("Options...");
    option_menu_item.addActionListener(
    		new ActionListener() {
  	  public void actionPerformed(ActionEvent e) {
  	    JMenuItem source = (JMenuItem)(e.getSource());
  	    String s = source.getText();
  	    if (s.equals("Options...")) {
  	      showOptions();
  	    }
  	  }
  	});
    popup.add(option_menu_item);
    
    popup.addSeparator();
    
    JMenuItem copy_item = new JMenuItem("Copy");
    copy_item.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          Thread thread = new Thread(
            new Runnable() {
              public void run() {
                try {
                	report_pane.copy();
                } catch (Exception e) {
                  e.printStackTrace();
                  MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
                  "Unexpected error copying text.\n" +
                  "See the console for more information.", false);
                }
                RRFBrowserGUI.getRRFBrowserGUI().disableGlassPane();
              }
            }
          );
          RRFBrowserGUI.getRRFBrowserGUI().enableGlassPane();
          thread.start();
        }
      }
    );
    popup.add(copy_item);
    
    //Add listener to components that can bring up popup menus.
    PopupListener popup_listener = new PopupListener(popup);
    report_pane.addMouseListener(popup_listener);
       
    return full_panel;
  }

  private StringBuffer redrawTree(ContextNode parent) {
  	StringBuffer sb = new StringBuffer();
  	 	
  	Context parent_cxt = parent.getContext();
  	Atom parent_atom = parent_cxt.getAtom();
  	// if expanding node or node is already opened
  	if (expanded_hm.containsKey(parent_cxt.getPathToRootAsString()
        + parent_atom.getAui())) {
      if (parent.getTotalChildCount() == 0) {
        // image is blank space saver, so text is indented correctly
        sb.append(indent).append(
            "<a href=\"" + parent_cxt.getPathToRootAsString()
                + parent_atom.getAui() + "\"><img src = \"" + blank_url
                + "\" height=9 width=9 border=0></a>").append("&nbsp;").append(
            "<a href=\"CUI:" + parent_cxt.getConcept().getCui() + "\">"
                + parent_atom.getStr() + "</a>").append(line_end);
      } else {
        // Context Tree Tops node should not have a link to a cui
        if (parent_atom.getStr().equals("Context Tree Tops")) {
          sb.append(indent).append(
              "<a href=><img src = \"" + minus_url + "\" border=0</a>").append(
              "&nbsp;").append(parent_atom.getStr()).append(line_end);
        } else {
          sb.append(indent).append(
              "<a href=\"" + parent_cxt.getPathToRootAsString()
                  + parent_atom.getAui() + "\"><img src = \"" + minus_url
                  + "\" border=0</a>").append("&nbsp;").append(
              "<a href=\"CUI:" + parent_cxt.getConcept().getCui() + "\">"
                  + parent_atom.getStr() + "</a>").append(line_end);
        }
        //
        // Check against current max_results setting
        //
        int total_child_count = parent.getTotalChildCount();
        ContextNode[] children = getChildren(parent);
        // If child node is already opened, we don't want to make
        // max results query on parent when it is redrawn. User may have already
        // responded not to "NO" increase max children on parent.
        // Also, we don't want to query user for max results if
        // user is closing a node (!expanding).
        ContextNode first_child = null;
        boolean child_already_opened = true;        
        if (children != null && children.length > 0) {
          first_child = children[0];
          child_already_opened = (expanded_hm.containsKey(first_child
            .getContext().getPathToRootAsString()
            + first_child.getContext().getAtom().getAui()));
        } else {
          sb.append(indent).append(" - No contexts to browse").append(line_end);
        }

        if (total_child_count > getMaxResults() && !child_already_opened
            && expanding) {
          boolean query_result = MMSToolkit
              .confirmRequestWithOption(
                  null,
                  "<html><b>Maximum Number of Contexts (Children)?</b>\n\n"
                      + "This context has "
                      + total_child_count
                      + " children.\n"
                      + "The limit for the number of children that can be displayed is "
                      + getMaxResults() + ".\n"
                      + "Would you like to increase it to " + total_child_count
                      + " in order to display them all?", "max_tree_results");
          if (query_result) {
            setMaxResults(total_child_count);
            children = getChildren(parent);
          }
        }
        //
        // Draw children recursively
        //
        for (int i = 0; i < children.length; i++) {
          indent = indent.concat("&nbsp;&nbsp;");
          ContextNode this_child = children[i];
          sb.append(redrawTree(this_child));
          indent = indent.substring(12);
        }
      }
      // if closing node
    } else {
      if (parent.getTotalChildCount() == 0) {
        sb.append(indent).append(
            "<a href=\"" + parent_cxt.getPathToRootAsString()
                + parent_atom.getAui() + "\"><img src = \"" + blank_url
                + "\" height=9 width=9 border=0></a>").append("&nbsp;").append(
            "<a href=\"CUI:" + parent_cxt.getConcept().getCui() + "\">"
                + parent_atom.getStr() + "</a>").append(line_end);
      } else {
        sb.append(indent).append(
            "<a href=\"" + parent_cxt.getPathToRootAsString()
                + parent_atom.getAui() + "\"><img src = \"" + plus_url
                + "\" border=0></a>").append("&nbsp;").append(
            "<a href=\"CUI:" + parent_cxt.getConcept().getCui() + "\">"
                + parent_atom.getStr() + "</a>").append(line_end);
      }
    }

    return sb;
  }
  
  /**
   * Gets the children for the given {@link TreeNode}.
   * @param parent {@link TreeNode}
   * @return {@link TreeNode}[]
   */
  private ContextNode[] getChildren(ContextNode parent) {

    //
    // Look up children (we could cache this)
    //
    Context[] children = null;
    final ContextNode node = (ContextNode)parent;
    try {
    	if (parent.equals(header))
    		index_lookup.setMaxResults(200);
    	else
        index_lookup.setMaxResults(getMaxResults());
      children = index_lookup.findChildContexts(node.getContext());
    } catch (Exception ex) {
      ex.printStackTrace();
      MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
          "Unexpected error looking up concept.\n" +
          "The most likely explanation is that your subset is missing\n" +
          "an indexes/PTR_AUI_RELA_MRHIER.x index file. \n" +
          "See the console for more information.", false);
    }

    //
    // Build context wrappers from children
    //
    final List results = new ArrayList();
    final ContextNode[] nodes = new ContextNode[children.length];
    for (int i=0; i<children.length; i++) {
      try {
        nodes[i] = new ContextNode(children[i]);
        nodes[i].getContext().setAtom(
            index_lookup.findAtomByAui(
            nodes[i].getContext().getAtom().getAui()));
        nodes[i].getContext().setConcept(
        		index_lookup.findConceptByAui(
        		nodes[i].getContext().getAtom().getAui()));
        nodes[i].setTotalChildCount(index_lookup.countChildContexts(
            nodes[i].getContext()));
        // remove the "root (of * )" wrapper on UWDA and NCBI so that they
        // will sort in alphabetical order in the tree browser
        if (nodes[i].toString().startsWith("root (of ")) {
          String str = nodes[i].toString();
          str = str.substring(9, str.lastIndexOf(")"));
          nodes[i].getContext().getAtom().setStr(str);
        }
      } catch (IOException ioe) {
        ioe.printStackTrace();
        MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
            "Unexpected error looking the atom name for a context ancestor.\n" +
            "The most likely explanation is that your subset is missing\n" +
            "an indexes/AUI_MRCONSO.x index file. \n" +
            "See the console for more information.", false);
      }
    }

    //
    // Sort children
    //
    Arrays.sort(nodes);

    //
    // build tree nodes
    //
    for (int i = 0; i < nodes.length; i++) {
      results.add(nodes[i]);
    }

    //
    // Return children as tree nodes
    //
    return (ContextNode[])results.toArray(new ContextNode[0]);
  }


  /**
   * Initializes the tree with the root contexts.
   */
  private void initializeRootContexts() {
 	
    expanded_hm.clear();

    rect = report_pane.getVisibleRect();
    refresh();
    
  }

  /**
   * Resets the configuration back to default values. When used with a GUI
   * this method should change the data model behind the GUI and allow
   * <code>setPanelSelections</code> to actually update the GUI.
   */
  public void resetToDefaults() { }

  /**
   * Method returns all properties specific to the {@link Finder}.
   * @return {@link Properties} only those directly related to the finder.
   */
  public Properties getProperties() {
    Properties new_filter_props = new Properties();
    new_filter_props.setProperty("max_children",
      new Integer(max_children).toString());

    return new_filter_props;
  }

  /**
   * Method provides the {@link GUIConfigurable} with all properties specific to the
   * {@link Finder}.
   * @param p {@link Properties}
   */
  public void setProperties(Properties p) {
    Properties filter_props = p;
    if(filter_props.containsKey("max_children")) {
      setMaxResults(new Integer((String)filter_props.get(
        "max_children")).intValue());
    }
  }

  /**
   * Returns the name.
   * @return the name
   */
  public String getName() {
    return "Tree Browser";
  }
  
  /**
   * Returns the content type used for the
   * {@link JEditorPane} display.
   * @return content type
   */
  public String getContentType() {
    return "text/html";
  }
  /**
   * Presents an advanced options GUI to the user.
   */
  public void showOptions() {
    final TreeBrowserFinderOptionsGUI tree_browser_finder_gui =
        new TreeBrowserFinderOptionsGUI(this);
    tree_browser_finder_gui.setVisible(true);
  }

  /**
   * Indicates whether or not an options menu item should be supplied
   * for advanced configuration options.
   * @return <code>true</code> if so; <code>false</code> otherwise
   */
  public boolean hasAdvancedOptions() {
    return true;
  }

  /**
   * Handles opening of a source directory.
   * @param source_dir the source directory
   */
  public void subsetDirectoryOpened(File source_dir) {
    try {
      index_lookup =
          new RichMRFilesIndexLookup(user_configuration.getSourcePaths()[0]);
      initializeRootContexts();
    } catch (IOException ioe) {
      ioe.printStackTrace();
      MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
                             "Unexpected error opening subset directory. \n" +
                             "See the console for more details.", false);
    }
  }

  /**
   * Returns the maximum number of cuis that will be
   * returned as results of the search.
   * @return max number of result cuis
   */
  private int getMaxResults() {
    return max_children;
  }

   /**
   * Sets the maximum number of cuis that will be
   * returned as results of the search.
   * @param max_children max number of cuis that will be returned
   */
  public void setMaxResults(int max_children) {

    //
    // any node that has been expanded does calculation to indicate if
    // not enough nodes are showing or too many are showing
    // expand and collapse all nodes that need fixing
    //
    this.max_children = max_children;
    index_lookup.setMaxResults(max_children);
  }
  

    
  /**
   * Redraws the text portions of the {@link Finder}.
   */
  public void refresh() {
    final StringBuffer sb = new StringBuffer(10000);
    sb.append("<html>");
    sb.append("<head><style>A {text-decoration:none; color:black}</style></head>");
    if (getContentType().equals("text/html")) {
      sb.append("<body><pre style=\"font-family: " + font_family + "; font-size:" + font_point_size + "pt;  white-space:nowrap\">").append(line_end);
    }

    //
    // Configure tree and header node
    //
    final Context head_cxt = new ContextImpl();
    final Atom head_atom = new AtomImpl();
    head_atom.setAui("");
    head_cxt.setAtom(head_atom);
    final Concept head_concept = new ConceptImpl();
    head_concept.setCui("");
    head_cxt.setConcept(head_concept);
    head_cxt.setPtr("");
    head_cxt.setSource(Source.getSource("MTH"));
    head_atom.setStr("Context Tree Tops");
    header = new ContextNode(head_cxt);
    header.setTotalChildCount(1);
    // auto expand the header "Context Tree Tops" so user can
    // immediately see source roots
    expanded_hm.put(header.getContext().getPathToRootAsString()
				+ header.getContext().getAtom().getAui(), new Integer(0));
    
  	sb.append(redrawTree(header));
 	
    if (getContentType().equals("text/html")) {
      sb.append("</pre>").append(line_end);
    }  	
  	
  	sb.append("</body></html>");
    report_pane.setText(sb.toString());
    
    // auto-scroll to appropriate place in text
    try {
      SwingUtilities.invokeAndWait(new Runnable() {
        public void run() {
          report_pane.scrollRectToVisible(rect);
        }
      });
    } catch (Exception e) {
      e.printStackTrace();
      MMSToolkit.reportError(RRFBrowserGUI.getRRFBrowserGUI(),
          "Unexpected error refreshing report.\n"
              + "See the console for more information.", false);
    }
  }

  //
  // Inner classes
  //
  public class PopupListener extends MouseAdapter {
    JPopupMenu popup;

    PopupListener(JPopupMenu popupMenu) {
      popup = popupMenu;
    }

    public void mousePressed(MouseEvent e) {
      maybeShowPopup(e);
    }

    public void mouseReleased(MouseEvent e) {
      maybeShowPopup(e);
    }

    private void maybeShowPopup(MouseEvent e) {
      if (e.isPopupTrigger()) {
        popup.show(e.getComponent(),
                   e.getX(), e.getY());
      }
    }
  }

  /**
   *  Used as a wrapper for {link Context}s to provide
   *  the toString() method that shows the atom string.
   */
  private class ContextNode  implements Comparable {

    private Context cxt = null;
    private int child_ct = 0;

    /**
     * Instantiates a {@link ContextNode}.
     * @param context the {@link Context} to wrap
     */
    public ContextNode(Context context) {
      cxt = context;
    }

    /**
     * Instantiates an empty {@link ContextNode}.
     */
    public ContextNode() { }

    /**
     * Returns the {@link Context}.
     * @return the {@link Context}
     */
    public Context getContext() {
      return cxt;
    }

    /**
     * Returns the child count.
     * @return the child count
     */
    public int getTotalChildCount() {
      return child_ct;
    }

    /**
     * Sets the child count.
     * @param child_ct the child count
     */
    public void setTotalChildCount(int child_ct) {
      this.child_ct = child_ct;
    }

    /**
     * Returns a {@link String} representation.
     * @return a {@link String} representation
     */
    public String toString() {
      return getContext().getAtom().getStr();
    }

    /**
     * Compares this {@link ContextNode} with the specified
     * {@link Object}.  Used for sort order.
     * @param o the {@link Object} to compare to
     * @return an <code>int</code> indicating the relative sort ordering
     */
    public int compareTo(Object o) {
      if(o instanceof ContextNode)
        return toString().compareToIgnoreCase(
          ((ContextNode)o).toString());
      else
        return 0;
    }

  } // end inner class


  /**
   * Options GUI for {@link TreeBrowserFinder}.
   *
   * @author Deborah Shapiro
   */
  public class TreeBrowserFinderOptionsGUI extends JDialog implements ActionListener {

    //
    // Private Fields
    //
    private JTextField max_children = new JTextField(4);

    //
    // finder
    //
    private TreeBrowserFinder finder;

    //
    // Constructor
    //
    /**
     * Instantiates a TreeBrowserFinderOptionsGUI
     * @param f the {@link TreeBrowserFinder}
     */
    public TreeBrowserFinderOptionsGUI(TreeBrowserFinder f) {
      super(RRFBrowserGUI.getRRFBrowserGUI());
      finder = f;
      setModal(true);
      setTitle("Tree Browser Options");
      initialize();
    };

    //
    // Methods
    //

    /**
     * Initialize the GUI.
     */
    private void initialize() {

      getContentPane().setLayout(new BorderLayout());

      final GridLayout data_layout = new GridLayout(2,2,8,8);
      final JPanel data_panel = new JPanel(data_layout);

      final FlowLayout done_layout = new FlowLayout();
      done_layout.setVgap(10);
      final JPanel done_panel = new JPanel(done_layout);
      final JPanel complete_panel = new JPanel(new BorderLayout());

      // configure data_panel
      final JLabel max_children_label = new JLabel("Maximum Number of Children:");
      data_panel.add(max_children_label);
      max_children.setText(new Integer(finder.getMaxResults()).toString());
      data_panel.add(max_children);

      // configure done_panel
      final JButton jdone = new JButton("Done");
      jdone.addActionListener(this);
      done_panel.add(jdone);

      // Create an action object
      Action cancelAction = new AbstractAction("Cancel") {
        {
          // Set action command to "Cancel"
          putValue(Action.ACTION_COMMAND_KEY, getValue(Action.NAME));
        }

        // This method is called when the action is invoked
        public void actionPerformed(ActionEvent e) {
          dispose();
        }
      };
      final JButton jcancel = new JButton(cancelAction);
      // Register keystroke
      done_panel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
          KeyStroke.getKeyStroke("ESCAPE"), cancelAction.getValue(Action.NAME));
      // Register action
      done_panel.getActionMap().put(cancelAction.getValue(Action.NAME),
          cancelAction);
      done_panel.add(jcancel);

      // Add the sub-panels to complete_panel
      complete_panel.add(data_panel, BorderLayout.CENTER);
      complete_panel.add(done_panel, BorderLayout.SOUTH);
      final EtchedBorder etched_border = new EtchedBorder(EtchedBorder.RAISED);
      complete_panel.setBorder(new CompoundBorder(etched_border,
                                                  new EmptyBorder(10, 10, 10, 10)));

      getContentPane().add(complete_panel, BorderLayout.CENTER);
      getRootPane().setDefaultButton(jdone);
      this.pack();
      this.setLocationRelativeTo(null);
    };

      // implementation of ActionListener interface
		public void actionPerformed(ActionEvent e) {
			if (e.getActionCommand().equals("Done")) {
				finder.setMaxResults(new Integer(max_children.getText()).intValue());

				TreeBrowserFinderOptionsGUI.this.dispose();
			}
		}
    

  } // End of TreeBrowserFinderOptionsGUI class


}


