/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms.filters;
 * Interface:  Filter.java
 *
 *************************************************************************/
package gov.nih.nlm.mms.filters;

import gov.nih.nlm.mms.GUIConfigurable;
import gov.nih.nlm.mms.SubsetListener;
import gov.nih.nlm.umls.meta.Concept;

/**
 *
 * This is an interface with methods used to
 * remove CUIs from the release.
 *
 * @author Deborah Shapiro
 *
 **/

public interface Filter extends GUIConfigurable, SubsetListener {

  //
  // Methods
  //

  /**
   * Method which removes a given {@link Concept} or its components
   * from the release.  Called during the subsetting process.
   * @deprecated Use one of the sub-interfaces instead
   * @param concept {@link Concept}
   */
  public void applyFilter(Concept concept);

  /**
   * Indicates if {@link Filter} can use the efficient mode during
   * the "Initialize Cuis" phase of the subsetting process.
   * The efficient mode reads only MRCONSO and MRSTY during the
   * "Initialize Cuis" phase.
   * @return <code>true</code> if efficient mode is acceptable
   *         <code>false</code> if efficient mode is not acceptable
   */
  public boolean isEfficientEnabled();
  
  /**
   * Returns a {@link String}[] of warnings accumulated during
   * processing of the {@link Filter}.
   * @return {@link String}[] of warnings
   */
  public String[] getWarnings();
}

