/**************************************************************************
 *
 * Package:	gov.nih.nlm.mms.filters
 * Object:	SemanticTypesToRemoveJTable.java
 *
 **************************************************************************/

package gov.nih.nlm.mms.filters;

import gov.nih.nlm.swing.SortableJTable;
import gov.nih.nlm.umls.meta.HierarchicalSemanticType;

import java.util.List;

import javax.swing.ListSelectionModel;

/**
 * Provides a table for the Semantic Types To Exclude panel that displays all
 * the Semantic Types and highlights the stys that will be excluded.
 * 
 * @author Deborah Shapiro, Brian Carlsen
 */

public class SemanticTypesToRemoveJTable extends SortableJTable {

  //
  // Model and list selection listener
  //
  private SemanticTypesToRemoveTableModel sty_model;

  private SemanticTypesToRemoveListSelectionListener sty_lsl = null;

  /**
   * Instantiates a {@link SemanticTypesToRemoveJTable} from the specified
   * {@link SemanticTypesToRemoveFilter}.
   * 
   * @param filter
   *          the {@link SemanticTypesToRemoveFilter}
   */
  public SemanticTypesToRemoveJTable(SemanticTypesToRemoveFilter filter) {
    super();
    sty_model = new SemanticTypesToRemoveTableModel();
    super.setModel(sty_model);
    getColumnModel().getColumn(0).setPreferredWidth(340);
    getColumnModel().getColumn(1).setPreferredWidth(115);
    getColumnModel().getColumn(2).setPreferredWidth(80);
    setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    final ListSelectionModel rowSM = this.getSelectionModel();
    sty_lsl = new SemanticTypesToRemoveListSelectionListener(filter, this);
    rowSM.addListSelectionListener(sty_lsl);
  }

  //
  // Methods
  //

  /**
   * Returns the {@link HierarchicalSemanticType} object at the given row
   * number.
   * 
   * @param row
   *          <code>int</code>
   * @return {@link HierarchicalSemanticType}
   */
  public HierarchicalSemanticType getSemanticTypeAtRow(int row) {
    return sty_model.getSemanticTypeAtRow(mapIndex(row));
  }

  /**
   * Returns the row number in the Semantic Types To Exclude table where a given
   * {@link HierarchicalSemanticType} is located.
   * 
   * @param sty
   *          {@link HierarchicalSemanticType}
   * @return <code>int</code> row
   */
  public int getRowForSemanticType(HierarchicalSemanticType sty) {
    return (reverseMapIndex(sty_model.getRowForSemanticType(sty)));
  }

  /**
   * Sets selections on the {@link SemanticTypesToRemoveJTable}
   * 
   * @param stys
   *          {@link List}
   */
  public void setSelections(List stys) {
    sty_lsl.setSelections(stys);
  }

}
