/**************************************************************************
 *
 * Package:    gov.nih.nlm.mms.filters;
 * Interface:  SourcesToRemoveOptionsGUI.java
 *
 *************************************************************************/
package gov.nih.nlm.mms.filters;

import gov.nih.nlm.mms.Dependency;
import gov.nih.nlm.mms.MMSToolkit;
import gov.nih.nlm.swing.FontSizeManager;
import gov.nih.nlm.swing.ListDialog;
import gov.nih.nlm.swing.SortableJTable;
import gov.nih.nlm.umls.meta.Source;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

/**
 * Sources To Remove options dialog box for user to set advanced options.
 * 
 * @author Deborah Shapiro
 */

public class SourcesToRemoveOptionsGUI extends JDialog {

  //
  // Private Fields
  //
  private JCheckBox enforce_family_selection = new JCheckBox(
      "Enforce Family Selection.", true);

  private JCheckBox enforce_dependent_source_selection = new JCheckBox(
      "Enforce Dependent Source Selection", false);

  private IPTableModel ipmodel;

  private SortableJTable jip_associations;

  private SourcesToRemoveFilter filter;

  public static final java.awt.Cursor wait_cursor = new java.awt.Cursor(
      java.awt.Cursor.WAIT_CURSOR);

  public static final java.awt.Cursor default_cursor = new java.awt.Cursor(
      java.awt.Cursor.DEFAULT_CURSOR);

  public static final MouseListener NULL_MOUSE_LISTENER = new MouseAdapter() {
  };

  private List source_dependencies = new ArrayList();

  //
  // Constructor
  //
  /**
   * Default constructor.
   * 
   * @param f
   *          {@link SourcesToRemoveFilter}
   */
  public SourcesToRemoveOptionsGUI(SourcesToRemoveFilter f) {
    super(MMSToolkit.getGui());
    filter = f;
    setModal(true);
    setTitle("Advanced Source List Options");
    initialize();
  };

  //
  // Methods
  //
  /**
   * This method sets up the GUI
   */
  private void initialize() {
    ipmodel = new IPTableModel();

    source_dependencies = new ArrayList(filter.getSourceDependencies());
    jip_associations = new SortableJTable(ipmodel);

    DataChangeListener dcl = new DataChangeListener();

    getContentPane().setLayout(new BorderLayout());
    JPanel data_panel = new JPanel(new GridBagLayout());
    JPanel table_panel = new JPanel(new GridBagLayout());
    JPanel done_panel = new JPanel(new GridBagLayout());
    JPanel complete_panel = new JPanel(new BorderLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.fill = GridBagConstraints.BOTH;
    constraints.insets = new Insets(2, 2, 2, 2);

    constraints.gridy = 0;
    constraints.gridwidth = 3;
    constraints.gridy++;
    enforce_family_selection.addActionListener(dcl);
    enforce_family_selection.setSelected(filter.enforceFamilySelection());
    data_panel.add(enforce_family_selection, constraints);
    constraints.gridy++;
    enforce_dependent_source_selection.addActionListener(dcl);
    enforce_dependent_source_selection.setSelected(filter
        .enforceDependentSourceSelection());
    data_panel.add(enforce_dependent_source_selection, constraints);

    /*
     * ButtonGroup group = new ButtonGroup(); group.add(remove_sources);
     * group.add(keep_sources);
     * 
     * constraints.gridy++;
     * remove_sources.setSelected(filter.removeSelectedSources());
     * remove_sources.addActionListener(dcl); data_panel.add(remove_sources,
     * constraints);
     * 
     * constraints.gridy++;
     * keep_sources.setSelected(!filter.removeSelectedSources());
     * keep_sources.addActionListener(dcl); data_panel.add(keep_sources,
     * constraints);
     */

    // configure table_panel
    TitledBorder titledBorder = new TitledBorder(new EtchedBorder(
        EtchedBorder.RAISED), "Manage Associations");
    titledBorder.setTitleJustification(TitledBorder.CENTER);
    table_panel.setBorder(new CompoundBorder(titledBorder, new EmptyBorder(10,
        10, 10, 10)));
    constraints.gridy = 0;
    constraints.gridwidth = 3;
    constraints.gridy++;
    JLabel jip_assoc_label = new JLabel("Dependent Source Associations");
    table_panel.add(jip_assoc_label, constraints);
    constraints.gridy++;
    jip_associations
        .setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

    JScrollPane jscroll = new JScrollPane(jip_associations);
    jscroll.setPreferredSize(new Dimension(300, 100));

    table_panel.add(jscroll, constraints);
    constraints.gridy++;

    constraints.gridx = 0;
    constraints.gridwidth = 1;
    constraints.weightx = 1.0;
    constraints.gridy++;
    JButton jclear = new JButton("Clear");
    jclear.addActionListener(dcl);
    table_panel.add(jclear, constraints);
    constraints.gridx++;
    JButton jdelete = new JButton("Delete");
    jdelete.addActionListener(dcl);
    table_panel.add(jdelete, constraints);
    constraints.gridx++;
    JButton jadd = new JButton("Add");
    jadd.addActionListener(dcl);
    table_panel.add(jadd, constraints);

    // configure done_panel
    constraints.gridx = 0;
    constraints.gridwidth = 1;
    constraints.weightx = 1.0;
    constraints.gridy++;
    JButton jdone = new JButton("Done");
    jdone.addActionListener(dcl);

    done_panel.add(jdone, constraints);

    // Add the sub-panels to complete_panel
    complete_panel.add(data_panel, BorderLayout.NORTH);
    complete_panel.add(table_panel, BorderLayout.CENTER);
    complete_panel.add(done_panel, BorderLayout.SOUTH);
    EtchedBorder etched_border = new EtchedBorder(EtchedBorder.RAISED);
    complete_panel.setBorder(new CompoundBorder(etched_border, new EmptyBorder(
        10, 10, 10, 10)));

    getRootPane().setDefaultButton(jdone);
    getContentPane().add(complete_panel, BorderLayout.CENTER);
    FontSizeManager.adjustFontSize(this);
    this.pack();
    this.setLocationRelativeTo(null);
  };

  private void startWaitMode() {
    setCursor(wait_cursor);
    getGlassPane().addMouseListener(NULL_MOUSE_LISTENER);
  }

  private void stopWaitMode() {
    setCursor(default_cursor);
    getGlassPane().removeMouseListener(NULL_MOUSE_LISTENER);
  }

  //
  // Inner classes
  //
  public class DataChangeListener implements java.awt.event.ActionListener {

    // constructor
    public DataChangeListener() {
    };

    // implementation of ActionListener interface
    public void actionPerformed(ActionEvent e) {
      String source_name, dependent_source_name;
      if (e.getActionCommand().equals("Clear")) {
        source_dependencies.clear();
        ipmodel.refresh();
      } else if (e.getActionCommand().equals("Done")) {
        filter.setEnforceFamilySelection(enforce_family_selection.isSelected());
        filter
            .setEnforceDependentSourceSelection(enforce_dependent_source_selection
                .isSelected());
        // check if remove/keep selections changed
        // if so, display the Source List filter tab
        /*
         * if ((filter.removeSelectedSources() && !remove_sources.isSelected()) ||
         * (!filter.removeSelectedSources() && remove_sources.isSelected()))
         * MMSToolkit.getGui().selectTabForFilter(filter);
         * filter.setRemoveSelectedSources(remove_sources.isSelected());
         * if(remove_sources.isSelected()) {
         * MMSToolkit.getGui().resetTitleForFilter(filter.getPanel(), "Sources
         * to Exclude"); //MMSToolkit.getGui().resetTabTitleForFilter(filter,
         * "Sources To Exclude"); } else {
         * MMSToolkit.getGui().resetTitleForFilter(filter.getPanel(), "Sources
         * to Keep"); //MMSToolkit.getGui().resetTabTitleForFilter(filter,
         * "Sources To Keep"); }
         * MMSToolkit.getGui().resetCommentForFilter(filter,
         * filter.getTabDescription());
         */

        filter.setSourceDependencies(source_dependencies);
        filter.fireNotUndoableDataChanged();
        SourcesToRemoveOptionsGUI.this.dispose();
      } else if (e.getActionCommand().equals("Delete")) {
        int[] selected = jip_associations.getSelectedRows();
        Source[][] selected_sources = new Source[selected.length][2];
        for (int i = 0, j = 0; i < jip_associations.getRowCount(); i++) {
          if (jip_associations.isRowSelected(i) == true) {
            selected_sources[j][0] = Source.getSource((String) jip_associations
                .getValueAt(selected[j], 0));
            selected_sources[j][1] = Source.getSource((String) jip_associations
                .getValueAt(selected[j], 1));
            j++;
          }
        }
        for (int i = 0; i < selected_sources.length; i++) {
          source_dependencies.remove(new Dependency(selected_sources[i][0],
              selected_sources[i][1]));
          ipmodel.refresh();
        }
      } else if (e.getActionCommand().equals("Add")) {
        String source_msg = "Enter the Source Name";
        Object[] source_list = new Object[filter.getSources().size()];
        Iterator iter = filter.getSources().iterator();
        int i = 0;
        while (iter.hasNext()) {
          Source s = (Source) iter.next();
          source_list[i++] = s.getVersionedSourceAbbreviation();
        }
        Arrays.sort(source_list);
        source_name = (String) ListDialog.showListSingleMode(
            SourcesToRemoveOptionsGUI.this, source_msg, "Source Name",
            source_list, (Object) source_list[0]);
        source_msg = "Enter the Dependent Source Name";
        if (source_name != null) {
          dependent_source_name = (String) ListDialog.showListSingleMode(
              SourcesToRemoveOptionsGUI.this, source_msg, "Source Name",
              source_list, (Object) source_list[0]);
          if (source_dependencies
              .contains(new Dependency(Source.getSource(source_name), Source
                  .getSource(dependent_source_name)))) {
            MMSToolkit.reportError(
                "This source and dependent source are already in the list.",
                false);
          } else if (source_name.equals(dependent_source_name)) {
            MMSToolkit.reportError(
                "The source and dependent source must be different.", false);
          } else if (!source_name.equals("") && (dependent_source_name != null)
              && !dependent_source_name.equals("")) {
            startWaitMode();
            source_dependencies.add(new Dependency(Source
                .getSource(source_name), Source
                .getSource(dependent_source_name)));
            ipmodel.refresh();
            SwingUtilities.invokeLater(new Runnable() {
              public void run() {
                stopWaitMode();
              }
            });
          }
        }
      }
    }
  }

  class IPTableModel extends AbstractTableModel {

    final String[] columnNames = { "Source", "Dependent Source" };

    public IPTableModel() {
    }

    public void refresh() {
      fireTableChanged(new TableModelEvent(IPTableModel.this, 0, getRowCount(),
          TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
    }

    public int getColumnCount() {
      return columnNames.length;
    }

    public int getRowCount() {
      return source_dependencies.size();
    }

    public String getColumnName(int col) {
      return columnNames[col];
    }

    public Object getValueAt(int row, int col) {
      Dependency d = (Dependency) (source_dependencies.get(row));
      if (col == 0)
        return d.getSource().getVersionedSourceAbbreviation();
      else
        return d.getDependentSource().getVersionedSourceAbbreviation();
    }

    public boolean isCellEditable(int row, int col) {
      return false;
    }

    public Class getColumnClass(int c) {
      return String.class;
    }
  };

}// End of SourcesToRemoveOptionsGUI class

