/**************************************************************************
 * Package:     gov.nih.nlm.umls.io
 * Interface:   ConceptOutputStream.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.io;

import gov.nih.nlm.umls.meta.Concept;

import java.io.IOException;

/**
 * Generically represents a way of writing {@link Concept}s out.
 * The output can be text files, text files plus a database load script,
 * or pretty much anything else.
 *
 * @author Deborah Shapiro
 */
public interface ConceptOutputStream {

  /**
   * Opens the output stream.  Here is where the actual files or database for
   * the output Metathesaurus should be opened.
   * @param output_loc locator information for the output stream
   * @throws IOException if there is a problem opening files
   */
  public void open(String output_loc) throws IOException;

  /**
   * Closes the output stream.  Here is where the actual files or database for
   * the output Metathesaurus should be closed.
   * @throws IOException if there is a problem closing files
   */
  public void close() throws IOException;

  /**
   * Write a {@link Concept} to the output stream.
   * @param concept a {@link Concept} to write to the output stream
   * @throws IOException if there is a problem reading the concept
   */
  public void writeConcept(Concept concept) throws IOException;

  /**
   * Returns a {@link String}[] of warnings accumulated during
   * processing of the output stream.
   * @return {@link String}[] of warnings
   */
  public String[] getWarnings();
}

