/***************************************************************************
 * Package:     gov.nih.nlm.umls.io
 * Interface:   PushBackReader.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.io;

import java.io.BufferedReader;
import java.io.IOException;

/**
 *
 * Extends {@link BufferedReader} to provide the ability to
 * push a single line of input back onto the reader to read it again
 * on the next <code>readLine()</code> call.
 *
 * @author Brian Carlsen
 */
public class PushBackReader extends BufferedReader {

  //
  // Fields
  //
  protected String line;

  /**
   * Instantiates a new {@link PushBackReader} wrapped around the
   * specified reader.
   * @param r {@link BufferedReader}
   */
  public PushBackReader(BufferedReader r) {
    super(r);
  }

  /**
   * Returns the next line from the reader.
   * @return the next line from the reader
   * @throws IOException
   */
  public String readLine() throws IOException {
    if (line != null) {
      String l = line;
      line = null;
      return l;
    }
    else
      return super.readLine();
  }

  /**
   * Push a line of input back onto the reader.
   * @param line input line to be pushed back on reader
   */
  public void push(String line) {
    this.line = line;
  }

}
