/**************************************************************************
 * Package:     gov.nih.nlm.umls.meta
 * Interface:   Attribute.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.meta;

/**
 * Represents information about an attribute attached
 * to an {@link Atom}, a {@link Concept}, or a {@link Relationship}.
 * This interface is the superclass for {@link RelationshipAttribute} and {@link AtomAttribute}.
 * Implementations of this interface should not be constructed.  Rather,
 * implementations of {@link RelationshipAttribute} or {@link AtomAttribute} should be constructed.
 * <p>
 * Following is a UML Class diagram showing the neighborhood around {@link Attribute}
 * <p><center><img src="Attribute.gif"></center>
 * <p>
 * All data in a row of the <code>MRSAT</code> file is represented here.
 * <p>
 * @author Brian Carlsen, Deborah Shapiro
 */
public interface Attribute
  extends AttributeElement, Comparable, ConceptElement,
  ContentViewElement, SourceIdentifierElement, SourceElement, SuppressibleElement {


  /**
   * Returns the {@link AttributeName} (ATN).
   * @return the {@link AttributeName}
   */
  public AttributeName getAttributeName();

  /**
   * Returns the {@link AttributeName} (ATN).
   * This is shorthand for {@link #getAttributeName()}.
   * @return the {@link AttributeName}
   */
  public AttributeName getAtn();

  /**
   * Sets the {@link AttributeName} (ATN).
   * @param atn the {@link AttributeName}
   */
  public void setAttributeName(AttributeName atn);

  /**
   * Sets the {@link AttributeName} (ATN).
   * This is shorthand for {@link #setAttributeName(AttributeName)}.
   * @param atn the {@link AttributeName}
   */
  public void setAtn(AttributeName atn);

  /**
   * Returns the attribute value.
   * @return the attribute value
   */
  public String getAttributeValue();

  /**
   * Returns the attribute value (ATV).
   * This is shorthand for {@link #getAttributeValue()}.
   * @return the attribute value
   */
  public String getAtv();

  /**
   * Sets the attribute value (ATV).
   * @param atv the attribute value
   */
  public void setAttributeValue(String atv);

  /**
   * Sets the attribute value.
   * This is shorthand for {@link #setAttributeValue(String)}.
   * @param atv the attribute value
   */
  public void setAtv(String atv);

  /**
   * Returns a {@link String} key which can be used
   * by equals, compareTo, and hashCode methods.
   * @return a {@link String} composed from key {@link Atom} fields
   */
  public String getKeyFields();

}