/**************************************************************************
 * Package:     gov.nih.nlm.umls.meta
 * Interface:   AttributeElement.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.meta;

/**
 * Represents information about a data element with attribute
 * identifiers.
 *
 * @author Brian Carlsen, Deborah Shapiro
 */
public interface AttributeElement {

  /**
   * Returns the unique identifier (ATUI).
   * @return the ATUI
   */
  public String getAttributeIdentifier();

  /**
   * Returns the unique identifier (ATUI). This
   * is shorthand for {@link #getAttributeIdentifier()}.
   * @return the ATUI
   */
  public String getAtui();

  /**
   * Sets the unique identifier (ATUI).
   * @param atui the ATUI
   */
  public void setAttributeIdentifier(String atui);

  /**
   * Sets the unique identifier (ATUI).
   * This is shorthand for {@link #setAttributeIdentifier(String)}.
   * @param atui the ATUI
   */
  public void setAtui(String atui);

  /**
   * Returns the source asserted attribute identifier (SATUI).
   * @return the SATUI
   */
  public String getSourceAttributeIdentifier();

  /**
   * Returns the source asserted attribute identifier (SATUI).
   * This is shorthand for {@link #getSourceAttributeIdentifier()}.
   * @return the SATUI
   */
  public String getSatui();

  /**
   * Sets the source asserted attribute identifier (SATUI).
   * @param satui the SATUI
   */
  public void setSourceAttributeIdentifier(String satui);

  /**
   * Sets the source asserted attribute identifier (SATUI)
   * This is shorthand for {@link #setSourceAttributeIdentifier(String)}.
   * @param satui the SATUI
   */
  public void setSatui(String satui);

}