/**************************************************************************
 * Package:     gov.nih.nlm.umls.meta
 * Interface:   ComponentHistory.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.meta;

/**
 * Represents source-asserted history information.
 * <p>
 * All data in a row of the <code>MRHIST</code> file is represented here.
 * <p>
 * @author Brian Carlsen, Deborah Shapiro
 */
public interface ComponentHistory extends
  Comparable, ConceptElement, ContentViewElement, SourceElement {

  /**
   * Returns the release date or version number of a source (SVER).
   * @return the release date or version number of a source
   */
  public String getVersion();

  /**
   * Returns the release date or version number of a source (SVER).
   * This is shorthand for {@link #getVersion()}.
   * @return the release date or version number of a source
   */
  public String getSver();

  /**
   * Sets the release date or version number of a source (SVER).
   * @param sver the release date or version number of a source
   */
  public void setVersion(String sver);

  /**
   * Sets the release date or version number of a source (SVER).
   * This is shorthand for {@link #setVersion(String)}.
   * @param sver the release date or version number of a source
   */
  public void setSver(String sver);

  /**
   * Returns the change key which identifies what changed (CHANGEKEY).
   * @return the change key
   */
  public String getChangeKey();

  /**
   * Sets the change key to identify what changed (CHANGEKEY).
   * @param changekey the change key
   */
  public void setChangeKey(String changekey);

  /**
   * Returns the new value for what changed (CHANGEVALUE).
   * @return the change value
   */
  public String getChangeValue();

  /**
   * Sets the new value for what changed (CHANGEVALUE).
   * @param changeval the change value
   */
  public void setChangeValue(String changeval);

  /**
   * Returns the change type which identifies what changed (CHANGETYPE).
   * @return the change type
   */
  public String getChangeType();

  /**
   * Sets the change type to identify what changed (CHANGETYPE).
   * @param changetype the change type
   */
  public void setChangeType(String changetype);

  /**
   * Returns the reason for the change (REASON).
   * @return the reason
   */
  public String getReason();

  /**
   * Sets the reason for the change (REASON).
   * @param reason reason for the changed
   */
  public void setReason(String reason);

  /**
   * Returns the unique identifier (UI).
   * This is shorthand for {@link #getIdentifier()}.
   * @return the unique identifier
   */
  public String getUi();

  /**
   * Returns the unique identifier (UI).
   * @return the unique identifier
   */
  public String getIdentifier();

  /**
   * Sets the unique identifier (UI).
   * @param ui the new value
   */
  public void setIdentifier(String ui);

  /**
   * Sets the unique identifier (UI).
   * This is shorthand for {@link #setIdentifier(String)}.
   * @param ui the unique identifier
   */
  public void setUi(String ui);

  /**
   * Returns a {@link String} key which can be used
   * by equals, compareTo, and hashCode methods.
   * @return a {@link String} composed from key {@link Atom} fields
   */
  public String getKeyFields();
}