/**************************************************************************
 * Package:     gov.nih.nlm.umls.meta
 * Interface:   CoocurrenceGroup.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.meta;

import java.util.List;

/**
 * Represents a group of co-ocurring {@link Concept}s or {@link Atom}s.
 * <p>
 * Following is a UML Class diagram showing the neighborhood around {@link CoocurrenceGroup}
 * <p><center><img src="CoocurrenceGroup.gif"></center>
 * <p>
 * @author Brian Carlsen, Deborah Shapiro
 */
public interface CoocurrenceGroup extends
   AtomElement, Comparable, ConceptElement, RelatedAtomElement,
   RelatedConceptElement {

  /**
   * Returns the {@link CoocurrenceContext}s.
   * @return an array of {@link CoocurrenceContext}s
   */
  public CoocurrenceContext [] getCoocurrenceContexts();

  /**
   * Returns the {@link CoocurrenceContext}s as a {@link List}.
   * @return the {@link CoocurrenceContext}s as a {@link List}
   */
  public List getCoocurrenceContextsAsList();

  /**
   * Sets the coocurrence attribute value. This is a comma separated
   * list of name=value pairs.
   * @param coc_contexts the coocuurence attribute value
   */
  public void setCoocurrenceContexts(String coc_contexts);

  /**
   * Replaces the {@link CoocurrenceContext}s with those in the
   * specified array.
   * @param coc_contexts an array of {@link CoocurrenceContext}s
   */
  public void setCoocurrenceContexts(CoocurrenceContext [] coc_contexts);

  /**
   * Replaces the {@link CoocurrenceContext}s with those in the
   * specified {@link List}
   * @param coc_contexts a {@link List} of {@link CoocurrenceContext}s
   */
  public void setCoocurrenceContexts(List coc_contexts);

  /**
   * Adds the specified {@link CoocurrenceContext}.
   * @param coc_context the {@link CoocurrenceContext} to be added
   */
  public void addCoocurrenceContext(CoocurrenceContext coc_context);

  /**
   * Removes the specified {@link CoocurrenceContext}.
   * @param coc_context the {@link CoocurrenceContext} to be removed
   */
  public void removeCoocurrenceContext(CoocurrenceContext coc_context);

  /**
   * Removes all {@link CoocurrenceContext}s.
   */
  public void clearCoocurrenceContexts();

  /**
   * Returns a {@link String} key which can be used
   * by equals, compareTo, and hashCode methods.
   * @return a {@link String} composed from key {@link CoocurrenceGroup} fields
   */
  public String getKeyFields();
}