/**************************************************************************
 * Package:     gov.nih.nlm.umls.meta
 * Interface:   STYMetaDescription.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.meta;
import java.util.List;

/**
 * Represents metadata about semantic network semantic types.
 *
 * @author Brian Carlsen, Deborah Shapiro
 */
public interface STYMetaDescription extends Comparable{


  /**
   * Returns the unique identifier (UI).
   * This is shorthand for {@link #getIdentifier()}.
   * @return the unique identifier
   */
  public String getUi();

  /**
   * Returns the unique identifier (UI).
   * @return the unique identifier
   */
  public String getIdentifier();

  /**
   * Returns the name (STY).
   * This is shorthand for {@link #getName()}.
   * @return the name
   */
  public String getSty();

  /**
   * Returns the name (STY).
   * @return the name
   */
  public String getName();

  /**
   * Returns the tree number (STN).
   * This is shorthand for {@link #getTreeNumber()}.
   * @return the tree number
   */
  public String getStn();

  /**
   * Returns the tree number (STN).
   * @return the tree number
   */
  public String getTreeNumber();

  /**
   * Returns the definition (DEF).
   * This is shorthand for {@link #getDefinition()}.
   * @return the definition
   */
  public String getDef();

  /**
   * Returns the definition (DEF).
   * @return the definition
   */
  public String getDefinition();

  /**
   * Returns a semi-colon (;) separated list of Metathesaurus strings (EX)
   * which are examples of this semantic type.
   * This is shorthand for {@link #getExamplesAsString()}.
   * @return a value listing Metathesaurus examples
   */
  public String getEx();

  /**
   * Returns a {@link String} array of of Metathesaurus strings (EX)
   * which are examples of this semantic type.
   * @return an {@link String} array of Metathesaurus examples
   */
  public String[] getExamples();

  /**
   * Returns a semi-colon (;) separated list of Metathesaurus strings (EX)
   * which are examples of this semantic type.
   * This is shorthand for {@link #getExamplesAsString()}.
   * @return a value listing Metathesaurus examples
   */
  public String getExamplesAsString();

  /**
   * Returns a {@link List} array of of Metathesaurus strings (EX)
   * which are examples of this semantic type.
   * @return an {@link List} of Metathesaurus examples
   */
  public List getExamplesAsList();

  /**
   * Returns the usage note (UN).
   * This is shorthand for {@link #getUsageNote()}.
   * @return the usage note
   */
  public String getUn();

  /**
   * Returns the usage note (UN).
   * @return the usage note
   */
  public String getUsageNote();


  /**
   * Indicates whether or not this semantic type or its descendents allow
   * the non human flag.
   * @return <code>true</code> if allowed, <code>false</code> if not
   */
  public boolean allowsNonHumanFlag();

  /**
   * Returns the non human flag value (NH).
   * @return the non human flag value
   */
  public String getNH();

}