/**************************************************************************
 * Package:     gov.nih.nlm.umls.meta
 * Interface:   SuppressibleElement.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.meta;

/**
 * Represents a data element that can be suppressed.
 *
 * @author Brian Carlsen, Deborah Shapiro
 */
public interface SuppressibleElement {

  /**
   * Returns the suppressibility value (SUPPRESS).
   * @return the {@link Suppressible} value
   */
  public Suppressible getSuppress();

  /**
   * Indicates whether or not this element is suppressible.
   * @return <code>true</code> if suppressible,
   *         <code>false</code> if not suppressible
   */
  public boolean isSuppressible();

  /**
   * Sets the suppressibility value (SUPPRESS).
   * @param suppress the {@link Suppressible} value
   */
  public void setSuppress(Suppressible suppress);

  /**
   * Indicates whether or not this element is obsolete.
   * @return <code>true</code> if obsolete,
   *         <code>false</code> if not obsolete
   */
  public boolean isObsolete();

}