/**************************************************************************
 * Package:     gov.nih.nlm.umls.meta
 * Interface:   TermAttribute.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.meta;

import java.util.List;

/**
 * Represents information about attributes sharing the same {@link Term}.
 * <p>
 * Following is a UML Class diagram showing the neighborhood around {@link TermAttribute}
 * <p><center><img src="TermAttribute.gif"></center>
 * <p>
 * @author Brian Carlsen, Deborah Shapiro
 */
public interface TermAttribute extends Comparable, ConceptElement, TermElement {

  /**
   * Returns the term identifier (LUI).
   * This is shorthand for {@link #getTermIdentifier()}.
   * @return the LUI
   */
  public String getLui();

  /**
   * Sets the term identifier (LUI). This is shorthand
   * for {@link #setTermIdentifier(String)}.
   * @param lui the LUI
   */
  public void setLui(String lui);

  /**
   * Returns the term identifier (LUI).
   * @return the LUI
   */
  public String getTermIdentifier();

  /**
   * Sets the term identifier (LUI).
   * @param lui the LUI
   */
  public void setTermIdentifier(String lui);

  /**
   * Returns all {@link StringAttribute}s.
   * @return a array of {@link StringAttribute}s
   */
  public StringAttribute [] getStringAttributes();

  /**
   * Returns all {@link StringAttribute}s as a {@link List}.
   * @return a {@link List} of {@link StringAttribute}s
   */
  public List getStringAttributesAsList();

  /**
   * Adds the specified {@link StringAttribute}.
   * @param as the {@link StringAttribute} to be added
   */
  public void addStringAttribute(StringAttribute as);

  /**
   * Removes the specified {@link StringAttribute}.
   * @param av the {@link StringAttribute} to be removed
   */
  public void removeStringAttribute(StringAttribute av);

  /**
   * Removes all {@link StringAttribute}s.
   */
  public void clearStringAttributes();

  /**
   * Replaces the {@link StringAttribute}s with the ones in the specified array.
   * @param avs a {@link StringAttribute}<code>[]</code>
   */
  public void setStringAttributes(StringAttribute [] avs);

  /**
   * Replaces the {@link StringAttribute}s with the ones in the specified {@link List}.
   * @param avs a {@link List} of {@link StringAttribute}s
   */
  public void setStringAttributes(List avs);

  /**
   * Returns a {@link String} key which can be used
   * by equals, compareTo, and hashCode methods.
   * @return a {@link String} composed from key {@link TermAttribute} fields
   */
  public String getKeyFields();
}