/**************************************************************************
 * Package:     gov.nih.nlm.umls.meta.impl
 * Class:       AtomAttributeImpl.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.meta.impl;

import gov.nih.nlm.umls.meta.Atom;
import gov.nih.nlm.umls.meta.AtomAttribute;

/**
 * Implementation of the {@link AtomAttribute} interface.
 *
 * @author Brian Carlsen, Deborah Shapiro
 */
public class AtomAttributeImpl extends AttributeImpl implements AtomAttribute {

  //
  // Fields
  //
  private Atom atom;

  /**
   * Instantiate an empty {@link AtomAttribute}.
   */
  public AtomAttributeImpl() {};

  /**
   * This is a copy constructor for {@link AtomAttribute}.
   * @param att {@link AtomAttribute} to be copied
   */
  public AtomAttributeImpl(AtomAttribute att) {
    super(att);
    setAtom(att.getAtom());
  }

  /**
   * Returns the {@link Atom} this {@link AtomAttribute} is connected to.
   * @return the {@link Atom} this {@link AtomAttribute} is connected to
   */
  public Atom getAtom() {
    return atom;
  }

  /**
   * Sets the {@link Atom} this {@link AtomAttribute} is connected to.
   * @param atom  the {@link Atom} this {@link AtomAttribute} is connected to
   */
  public void setAtom(Atom atom) {
    this.atom = atom;
    key_fields_flag = false;
  }

  /**
   * Returns a {@link String} key which can be used
   * by equals, compareTo, and hashCode methods.
   * @return a {@link String} composed from the cui, lui, sui,
   * aui, versioned source, atui, atn, atv, satui, and code.
   */
  public String getKeyFields() {
    if (key_fields == null || !key_fields_flag) {
      final StringBuffer sb1 = new StringBuffer(100);
      sb1.append(getConcept() != null ? getConcept().getCui() : "")
      .append(getAtom() != null ? getAtom().getSui() : "")
      .append(getAtom() != null ? getAtom().getAui() : "")
      .append(getSource() != null ? getSource().getVsab() : "")
      .append(getAtom() != null ? getAtom().getCode() : "")
      .append(getAtui())
      .append(getAtn() != null ? getAtn().getAbbreviation() : "")
      .append(getSatui());
      key_fields = sb1.toString();
      key_fields_flag = true;
    }
    return key_fields;
  }

  /**
   * Return a hash code based on key fields.
   * @return a hash code based on key fields
   */
  public int hashCode() {
    return getKeyFields().hashCode();
  }

  /**
   * Returns <code>true</code> if the specified object is
   * equal to this object.  The specified object must be an
   * {@link AtomAttribute} with matching key fields.
   * @param o an {@link Object} to compare against
   * @return boolean <code>true</code> if Object o is equal
   *                 <code>false</code> otherwise
   */
  public boolean equals(Object o) {
    if (o == null || !(o instanceof AtomAttribute)) return false;
    return getKeyFields().equals(((AtomAttribute)o).getKeyFields());
  }

  /**
   * Compares this {@link AtomAttribute} to another one.  Results
   * are indeterminate if the specified parameter is not of the
   * same class.
   * @param o the {@link Object} to compare to
   * @return <code>int</code> representing the relative ordering
   *         of this and the compare-to object
   */
  public int compareTo(Object o) {
    if (o instanceof AtomAttribute)
      return getKeyFields().compareTo(((AtomAttribute)o).getKeyFields());
    else
      return 0;
  }

}
