/**************************************************************************
 * Package:     gov.nih.nlm.umls.meta.impl
 * Class:       RelationshipAttributeImpl.java
 *
 **************************************************************************/
package gov.nih.nlm.umls.meta.impl;

import gov.nih.nlm.umls.meta.Relationship;
import gov.nih.nlm.umls.meta.RelationshipAttribute;

/**
 * Implementation of the {@link RelationshipAttribute} interface.
 *
 * @author Brian Carlsen, Deborah Shapiro
 */
public class RelationshipAttributeImpl extends AttributeImpl implements RelationshipAttribute {

  //
  // Fields
  //
  private Relationship rel;

  /**
   * Instantiate an empty {@link RelationshipAttribute}.
   */
  public RelationshipAttributeImpl() {};

  /**
   * This is a copy constructor for {@link RelationshipAttribute}.
   * @param rui_a {@link RelationshipAttribute} to be copied
   */
  public RelationshipAttributeImpl(RelationshipAttribute rui_a) {
    super(rui_a);
    setRelationship(rui_a.getRelationship());
  }

  /**
   * Returns the {@link Relationship} this {@link RelationshipAttribute} is connected to.
   * @return the {@link Relationship} this {@link RelationshipAttribute} is connected to
   */
  public Relationship getRelationship() {
    return rel;
  }

  /**
   * Sets the {@link Relationship} this {@link RelationshipAttribute} is connected to.
   * @param rel  the {@link Relationship} this {@link RelationshipAttribute} is connected to
   */
  public void setRelationship(Relationship rel) {
    this.rel = rel;
    key_fields_flag = false;
  }

  /**
   * Returns a {@link String} key which can be used
   * by equals, compareTo, and hashCode methods.
   * @return a {@link String} composed from the cui, aui, sui,
   * versioned source, tty, and code.
   */
  public String getKeyFields() {
    if (key_fields == null || !key_fields_flag) {
      StringBuffer sb1 = new StringBuffer(200);
      sb1.append(getConcept() != null ? getConcept().getCui() : "")
         .append(getRelationship() != null ? getRelationship().getRui() : "")
         .append(getAtui())
         .append(getSatui());
      key_fields = sb1.toString();
      key_fields_flag = true;
    }
    return key_fields;
  }

  /**
   * Return a hash code based on key fields.
   * @return a hash code based on key fields
   */
  public int hashCode() {
    return getKeyFields().hashCode();
  }

  /**
   * Returns <code>true</code> if the specified object is
   * equal to this object.  The specified object must be a
   * {@link RelationshipAttribute} with matching key fields.
   * @param o an {@link Object} to compare against
   * @return boolean <code>true</code> if Object o is equal
   *                 <code>false</code> otherwise
   */
  public boolean equals(Object o) {
    if (o == null || !(o instanceof RelationshipAttribute)) return false;
    return getKeyFields().equals(((RelationshipAttribute)o).getKeyFields());
  }

  /**
   * Compares this {@link RelationshipAttribute} to another one.  Results
   * are indeterminate if the specified parameter is not of the
   * same class.
   * @param o the {@link Object} to compare to
   * @return <code>int</code> representing the relative ordering
   *         of this and the compare-to object
   */
  public int compareTo(Object o) {
    if (o instanceof RelationshipAttribute)
      return getKeyFields().compareTo(((RelationshipAttribute)o).getKeyFields());
    else
      return 0;
  }

}
